CREATE OR REPLACE PROCEDURE MXE_AP_SP
/*=========================================================================
* Licensed Materials - Property of IBM
*
* Restricted Materials of IBM
*
* 5724-M19
*
* (C) COPYRIGHT IBM CORP. 2008. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*==========================================================================
* Licensed Materials - Property of MRO Software
*
* Restricted Materials of MRO Software
*
* (C) COPYRIGHT MRO Software, Inc. 2001 - 2006. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with MRO Software, Inc.
*==========================================================================
*
* Filename:	 OAPX.SP
* Procedure: MXE_AP_SP
*
* Description:
*
*	Procedure to pass data from Maximo to Oracle Financials.
*
* Public Functions:
*
*
* Modification History
*
*   04-Jan-2002  KM   Created for New Open Invoice Interface of Oracle
*	18-Mar-2002	 KM   Changed the WONUM column to REFWO
*	14-Oct-2002	 KM   Added the functionality for Blanket Releases
*   06-May-2003  AKJ  Added Project information.
*   21-Aug-2003  AKJ  Issue_025468 - Use PONUM for Release ID lookup
*   06-Nov-2003  KM   Issue_10665 - Use LoadedCost instead of LineCost.
*	17-Dec-2004  KM   Upgraded to 6.0 Release
*   14-Sep-2005  AKJ  Issue_86171 - Mapped project accounting columns.
*   07-Nov-2008  HRF  Issue_08-23754 - MX 7.1 OA no longer sends Releases to Oracle
* 	02-Dec-2008  FG   Issue_08-25152 - MX 6.2 to 7.1 MEAORA UPG - Got 'MXE_AP_SP - Invalid Blanket Release in Oracle' when doing Invoice from a Release PO in Maximo.'
*========================================================================*/
(
	InvoiceRec IN OUT MXINVOICE_IFACE%RowType
) IS
	PrgName		          VARCHAR2(40) := ' MXE_AP_SP';
	ProcMsg		          VARCHAR2(100);
	OFAPHRec	          AP_INVOICES_INTERFACE%ROWTYPE;
	OFAPLRec	          AP_INVOICE_LINES_INTERFACE%ROWTYPE;
	vENTERBYID	          NUMBER;
	vVENDORID	          NUMBER;
	vVENDORSITEID         NUMBER;
	vCOAID		          NUMBER;
	vDEBITGLID	          NUMBER;
	vCREDITGLID	          NUMBER;
	Error_Status          NUMBER;
	vLine_only            BOOLEAN := FALSE;
    vPONUM			      MXINVOICE_IFACE.PONUM%TYPE;
    vlinenum              varchar2(15);
	vACCOUNT_TYPE	      GL_CODE_COMBINATIONS.ACCOUNT_TYPE%Type;
	v_por_attribute15     VARCHAR2(150);
	v_rel_id              NUMBER;
	skip_transaction 	  BOOLEAN;
	vORDERUNIT			  VARCHAR2(25);

BEGIN
	ProcMsg := ' Invoice:' || InvoiceRec.INVOICENUM;
	EnterProc(PrgName,ProcMsg);

--
--  6/13/00 - fix for issue 993190 - don't send $0 invoices to Oracle
--
--	If InvoiceRec.TOTALCOST = 0 Then
--	   return;
--	End if;

/*
*  Set the Operating Unit ID for  Oracle Financials.
*/
	SET_OPERID_SP(InvoiceRec.SITEID);
	
/*
*   Validate the Credit and Debit GL Accounts and pass the Code Combinations ID to the
*    interface table.
*/
	of_id_pkg.get_id_sp('COAID', InvoiceRec.ORGID, Null, vCOAID, Error_Status);
    IF Error_Status != 0 THEN
       InvoiceRec.OA_IfaceMessage := 'Invalid Set of Books ID:' || InvoiceRec.ORGID;
       MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IfaceMessage);
    END IF;

	MaxDebugApi.Output(' COAID: '||vCOAID);
	MaxDebugApi.Output(' APCONTROLACCT: '||InvoiceRec.APCONTROLACCT);
	MaxDebugApi.Output(' APSUSPENSEACCT: '||InvoiceRec.APSUSPENSEACCT);

	IF InvoiceRec.APCONTROLACCT IS NOT NULL THEN

		api_glcc_pkg.gl_usrcc_sp(vCOAID,
								 InvoiceRec.APCONTROLACCT,
		                         vCREDITGLID,
					             Error_Status);
	    MaxDebugApi.Output('GL Credit ID : '||vCREDITGLID);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'APCONTROLACCT is an invalid GL code.';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
	ELSE
			InvoiceRec.OA_IFACEMESSAGE := 'APCONTROLACCT MUST have a value!';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
	END IF;

	IF InvoiceRec.APSUSPENSEACCT IS NOT NULL THEN
		api_glcc_pkg.gl_usrcc_sp(vCOAID,
								 InvoiceRec.APSUSPENSEACCT,
		                         vDEBITGLID,
		                         Error_Status);
     	MaxDebugApi.Output('GL Debit ID : '||vDEBITGLID);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'APSUSPENSEACCT is an invalid GL code.';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
	ELSE
			InvoiceRec.OA_IFACEMESSAGE := 'APSUSPENSEACCT MUST have a value!';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
	END IF;

	IF vCREDITGLID IS NOT NULL THEN
	  	of_id_pkg.get_id_spr('ACCOUNT_TYPE',
				    vCREDITGLID,
					InvoiceRec.SITEID,
				    vACCOUNT_TYPE,
				    Error_Status);
	    MaxDebugApi.Output('GL Credit ID : '||vCREDITGLID);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'APCONTROLACCT is an invalid GL code.';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
	END IF;

	If InvoiceRec.ENTERBY is NOT NULL THEN
		of_id_pkg.get_id_sp('USRID', InvoiceRec.ENTERBY, InvoiceRec.SITEID, vENTERBYID, Error_Status);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'Modified By user '||InvoiceRec.ENTERBY||' is not valid in Oracle Financials';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
	End If;

	--
	-- Seperating Vendor from Operating Unit ID
	--
	IF INSTR(InvoiceRec.VENDOR,'-') != 0 THEN
		InvoiceRec.VENDOR := SUBSTR(InvoiceRec.VENDOR,INSTR(InvoiceRec.VENDOR,'-')+1,LENGTH(InvoiceRec.VENDOR));
		MaxDebugApi.Output('Vendor after operunitid is removed:'||InvoiceRec.VENDOR);
	End If;
	IF INSTR(InvoiceRec.VENDOR,'-') != 0 THEN
		of_id_pkg.get_id_sp('VNDID',
				    SUBSTR(InvoiceRec.VENDOR,0,INSTR(InvoiceRec.VENDOR,'-')-1),
	                InvoiceRec.SITEID,
				    vVENDORID,
				    Error_Status);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'Invalid vendor header';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
	    MaxDebugApi.Output(' TJS AT VEND SITE:'||InvoiceRec.VENDOR);

		of_id_pkg.get_id_sp('VNDSITEID',
				    InvoiceRec.VENDOR,
					InvoiceRec.SITEID,
				    vVENDORSITEID,
				    Error_Status);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'Invalid vendor site';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
	ELSE
		of_id_pkg.get_id_sp('VNDID',
				    InvoiceRec.VENDOR,
		            InvoiceRec.SITEID,
				    vVENDORID,
				    Error_Status);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'Invalid vendor';
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
	END IF;

/*
*	if a blanket release, use the original ponum
*
*	Issue #08-25152 - Author: FG - Date: 02/Dec/2008 - Removed blanket release from maximo section as PO outbound just send Std POs. 
*/
	If InvoiceRec.PONUM is not null Then
    	If InvoiceRec.OA_POTYPE = 'REL' and nvl(InvoiceRec.OA_OWNERSYSID,'x') = 'OA' then
       		-- this is a blanket release originated in Oracle
	      	vPONUM := Substr(InvoiceRec.PONUM,1,Instr(InvoiceRec.PONUM,'-')-1);
	   		MaxDebugApi.Output('vPONUM: '||vPONUM );
	   		OFAPLRec.RELEASE_NUM :=  SUBSTR(InvoiceRec.PONUM,INSTR(InvoiceRec.PONUM,'-')+1,LENGTH(InvoiceRec.PONUM));
		Else
			-- THIS IS A STD PO
	   		vPONUM := InvoiceRec.PONUM;
    	End If;
	End If;

   /*
	*	Get PO_HEADER_ID based on PONUM
	*/
	If vPONUM is not null Then
	  of_id_pkg.get_id_sp('POHEADERID',
			      vPONUM,
			      InvoiceRec.SITEID,
			      OFAPLRec.PO_HEADER_ID,
			      Error_Status);
	  If Error_Status != 0 then
		InvoiceRec.OA_IFACEMESSAGE := 'Invalid PO Number';
		MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
	  End If;
	End If;
	MaxDebugApi.Output('after PO id');


    /*  06-May-2003  Getting the project values through
	*      userdata column from Java ERP exit
    *   proj_values(1) = projectid
    *   proj_values(2) = taskid
    *   proj_values(3) = expenditem
    *   proj_values(4) = chargeorg
    */

/*
*   Set values.
*/

	OFAPHRec.INVOICE_NUM			  :=	InvoiceRec.INVOICENUM;
	OFAPHRec.INVOICE_TYPE_LOOKUP_CODE :=    Null;
	OFAPHRec.INVOICE_DATE			  :=	NVL(InvoiceRec.INVOICEDATE,SYSDATE);
	OFAPHRec.VENDOR_ID				  :=	vVENDORID;
	OFAPHRec.VENDOR_NUM      		  :=	Null;
	OFAPHRec.VENDOR_NAME     		  :=	Null;
	OFAPHRec.VENDOR_SITE_ID			  :=	vVENDORSITEID;
	OFAPHRec.VENDOR_SITE_CODE         :=    Null;
   	OFAPHRec.INVOICE_AMOUNT			  :=	InvoiceRec.TOTALCOST;

	IF InvoiceRec.CURRENCYCODE IS NULL THEN
        OFAPHRec.INVOICE_CURRENCY_CODE	:=	 InvoiceRec.OA_GLCURNCY;
	ELSE
        OFAPHRec.INVOICE_CURRENCY_CODE	:=	 InvoiceRec.CURRENCYCODE;
	END IF;

	OFAPHRec.EXCHANGE_RATE_TYPE	      :=	InvoiceRec.OA_GLCURNCYTYPE;
	OFAPHRec.EXCHANGE_RATE		      :=	InvoiceRec.EXCHANGERATE;
	OFAPHRec.EXCHANGE_DATE		      :=	NVL(InvoiceRec.EXCHANGEDATE,InvoiceRec.CHANGEDATE);
	OFAPHRec.TERMS_ID    			  :=    Null;
	OFAPHRec.TERMS_NAME  			  :=	Null;
	OFAPHRec.DESCRIPTION			  :=	InvoiceRec.INVOICEDESC;
	OFAPHRec.AWT_GROUP_ID     		  :=    Null;
	OFAPHRec.AWT_GROUP_NAME   		  :=	Null;
	OFAPHRec.LAST_UPDATE_DATE		  :=	NVL(InvoiceRec.CHANGEDATE,SYSDATE);
	OFAPHRec.LAST_UPDATED_BY		  :=	vENTERBYID;
	OFAPHRec.LAST_UPDATE_LOGIN		  :=	Null;
	OFAPHRec.CREATION_DATE			  :=	SYSDATE;
	OFAPHRec.CREATED_BY				  :=	vENTERBYID;
	OFAPHRec.ATTRIBUTE_CATEGORY		  :=	Null;
	OFAPHRec.ATTRIBUTE1			      :=	Null;
	OFAPHRec.ATTRIBUTE2			      :=	Null;
	OFAPHRec.ATTRIBUTE3			      :=	Null;
	OFAPHRec.ATTRIBUTE4			      :=	Null;
	OFAPHRec.ATTRIBUTE5			      :=	Null;
	OFAPHRec.ATTRIBUTE6			      :=	Null;
	OFAPHRec.ATTRIBUTE7			      :=	Null;
	OFAPHRec.ATTRIBUTE8			      :=	Null;
	OFAPHRec.ATTRIBUTE9			      :=	Null;
	OFAPHRec.ATTRIBUTE10			  :=	Null;
	OFAPHRec.ATTRIBUTE11			  :=	Null;
	OFAPHRec.ATTRIBUTE12			  :=	Null;
	OFAPHRec.ATTRIBUTE13			  :=	Null;
	OFAPHRec.ATTRIBUTE14			  :=	Null;
	OFAPHRec.ATTRIBUTE15			  :=	Null;
	OFAPHRec.STATUS					  :=	Null;
   	OFAPHRec.SOURCE					  :=	'MAXIMO';
	OFAPHRec.GROUP_ID                 :=	Null;
	OFAPHRec.REQUEST_ID               :=	Null;
	OFAPHRec.PAYMENT_CROSS_RATE_TYPE  :=	Null;
	OFAPHRec.PAYMENT_CROSS_RATE_DATE  :=	Null;
	OFAPHRec.PAYMENT_CROSS_RATE       :=	Null;
	OFAPHRec.PAYMENT_CURRENCY_CODE    :=	Null;
	OFAPHRec.WORKFLOW_FLAG            :=	Null;
	OFAPHRec.DOC_CATEGORY_CODE		  :=	Null;
	OFAPHRec.VOUCHER_NUM			  :=	Null;
	OFAPHRec.PAYMENT_METHOD_LOOKUP_CODE	 :=	  Null;
	OFAPHRec.PAY_GROUP_LOOKUP_CODE    :=	Null;
	OFAPHRec.GOODS_RECEIVED_DATE      :=	Null;
	OFAPHRec.INVOICE_RECEIVED_DATE    :=	Null;
	OFAPHRec.GL_DATE		          :=	NVL(InvoiceRec.INVOICEDATE,InvoiceRec.ENTERDATE);

	If vACCOUNT_TYPE = 'L' Then
	OFAPHRec.ACCTS_PAY_CODE_COMBINATION_ID	:=		vCREDITGLID;
	Else
	OFAPHRec.ACCTS_PAY_CODE_COMBINATION_ID	:=		Null;
	End If;

	OFAPHRec.USSGL_TRANSACTION_CODE	  :=	Null;


	/*
	* Setting Line Information
	*/
	OFAPLRec.LINE_NUMBER 		    :=		InvoiceRec.INVOICELINENUM;
	--
    --  Getting the Value of the Control APLINETYPE from Java ERP exit
	--
	OFAPLRec.LINE_TYPE_LOOKUP_CODE	:=	    InvoiceRec.OA_APLINETYPE;
	--
	-- set the line cost - set to unloaded line cost + any line taxes that are
	--                     marked as inclusive.
	--
	OFAPLRec.AMOUNT				      :=   InvoiceRec.LINECOST;
	OFAPLRec.ACCOUNTING_DATE	      :=   NVL(InvoiceRec.INVOICEDATE,InvoiceRec.ENTERDATE);
	OFAPLRec.DESCRIPTION              :=   InvoiceRec.DESCRIPTION;
    OFAPLRec.AMOUNT_INCLUDES_TAX_FLAG :=   Null;
	OFAPLRec.PO_NUMBER                :=   vPONUM;

	-- Cannot Include Tax for PO matched lines,
	-- having loadedCost only for non PO matched lines.
    If OFAPLRec.PO_NUMBER is null then
		OFAPLRec.AMOUNT			:=  InvoiceRec.LOADEDCOST;
		If InvoiceRec.Inclusive1 = 1 AND
		   InvoiceRec.TAX1 <> 0	 then
			--OFAPLRec.AMOUNT	:= OFAPLRec.AMOUNT + InvoiceRec.TAX1;
	        OFAPLRec.AMOUNT_INCLUDES_TAX_FLAG := 'Y';
		End If;
		If InvoiceRec.Inclusive2 = 1 AND
		   InvoiceRec.TAX2 <> 0	 then
			--OFAPLRec.AMOUNT	:= OFAPLRec.AMOUNT + InvoiceRec.TAX2;
	        OFAPLRec.AMOUNT_INCLUDES_TAX_FLAG := 'Y';
		End If;
		If InvoiceRec.Inclusive3 = 1 AND
		   InvoiceRec.TAX3 <> 0	 then
			--OFAPLRec.AMOUNT	:= OFAPLRec.AMOUNT + InvoiceRec.TAX3;
	        OFAPLRec.AMOUNT_INCLUDES_TAX_FLAG := 'Y';
		End If;
		If InvoiceRec.Inclusive4 = 1 AND
		   InvoiceRec.TAX4 <> 0	 then
			--OFAPLRec.AMOUNT	:= OFAPLRec.AMOUNT + InvoiceRec.TAX4;
	        OFAPLRec.AMOUNT_INCLUDES_TAX_FLAG := 'Y';
		End If;
		If InvoiceRec.Inclusive5 = 1 AND
		   InvoiceRec.TAX5 <> 0	 then
			--OFAPLRec.AMOUNT	:= OFAPLRec.AMOUNT + InvoiceRec.TAX5;
	        OFAPLRec.AMOUNT_INCLUDES_TAX_FLAG := 'Y';
		End If;
	ELSE
	    OFAPLRec.AMOUNT			:=  InvoiceRec.LINECOST;
	End If;

        -- Issue #08-23754 - MX 7.1 OA no longer sends Releases to Oracle
	-- Invoice is for an STD PO or CHANGE PO.
	If InvoiceRec.polinenum is not null and nvl(InvoiceRec.OA_OWNERSYSID,'x') = 'OA' Then
	    vlinenum := to_char(InvoiceRec.polinenum);
	    OFAPLRec.PO_LINE_NUMBER     := to_number(substr(vlinenum,2,4));
	    OFAPLRec.PO_SHIPMENT_NUM    := to_number(substr(vlinenum,6,4));
	    OFAPLRec.PO_DISTRIBUTION_NUM    := to_number(substr(vlinenum,10,4));
	Else
	    OFAPLRec.PO_LINE_NUMBER     := InvoiceRec.polinenum;
	End If;

	--
    -- Got error when importing with Item description,
    -- either polinenum should be given or the exact decsription of item in POLINE should be given.
    --
	If (OFAPLRec.PO_NUMBER is not null  and OFAPLRec.PO_LINE_NUMBER is not null) then
	OFAPLRec.ITEM_DESCRIPTION		:= Null;
	else
	OFAPLRec.ITEM_DESCRIPTION		:=		'Approved Maximo Invoice: '||InvoiceRec.INVOICENUM;
	End If;

    -- For the PO matched lines sending another line as line type 'MISCELLANEOUS'
    -- for the difference in the Amount (LOADEDCOST - LINECOST).
	If InvoiceRec.OA_APLINETYPE like 'MAXIMO MISCELLANEOUS' then
		   OFAPLRec.AMOUNT :=  InvoiceRec.LOADEDCOST - InvoiceRec.LINECOST;
		   OFAPLRec.LINE_TYPE_LOOKUP_CODE := 'MISCELLANEOUS';
		   OFAPLRec.ITEM_DESCRIPTION := Null;
		   -- Author: FG Date: 11/23/2007 Desc.: Commented to keep the correct line number comming from java class
		   -- OFAPLRec.LINE_NUMBER := null;
		   OFAPLRec.PO_NUMBER := null;
		   OFAPLRec.PO_LINE_NUMBER := null;
		   OFAPLRec.PO_SHIPMENT_NUM := null;
		   OFAPLRec.PO_DISTRIBUTION_NUM := null;
		   
		   -- Author: FG Date: 11/23/2007 Desc: New fields necessary to set miscelaneous invoice line
		   OFAPLRec.UNIT_PRICE := null;
		   OFAPLRec.QUANTITY_INVOICED := null;
	Else
		OFAPLRec.QUANTITY_INVOICED       	:=	 InvoiceRec.QUANTITY;
		OFAPLRec.UNIT_PRICE              	:=	 InvoiceRec.UNITCOST;
	End If;

	OFAPLRec.PRORATE_ACROSS_FLAG     	:=	 Null;
	OFAPLRec.TAX_CODE                	:=	 Null;
	OFAPLRec.FINAL_MATCH_FLAG        	:=	 Null;
	OFAPLRec.PO_HEADER_ID            	:=	 Null;
	OFAPLRec.PO_LINE_ID              	:=	 Null;
	OFAPLRec.PO_LINE_LOCATION_ID     	:=	 Null;
	OFAPLRec.PO_DISTRIBUTION_ID      	:=	 Null;
	OFAPLRec.PO_UNIT_OF_MEASURE      	:=	 Null;
	OFAPLRec.INVENTORY_ITEM_ID       	:=	 Null;
	OFAPLRec.SHIP_TO_LOCATION_CODE   	:=	 Null;
	OFAPLRec.DISTRIBUTION_SET_ID     	:=	 Null;
	OFAPLRec.DISTRIBUTION_SET_NAME   	:=	 Null;
	OFAPLRec.DIST_CODE_CONCATENATED  	:=	 Null;

	If (OFAPLRec.PO_NUMBER is not null  and OFAPLRec.PO_LINE_NUMBER is not null) then
	OFAPLRec.DIST_CODE_COMBINATION_ID	:=	 Null;
	else
	OFAPLRec.DIST_CODE_COMBINATION_ID	:=	 vDEBITGLID;
	End If;

	OFAPLRec.AWT_GROUP_ID            	:=	 Null;
	OFAPLRec.AWT_GROUP_NAME          	:=	 Null;
	OFAPLRec.LAST_UPDATED_BY		    :=	 vENTERBYID;
	OFAPLRec.LAST_UPDATE_DATE		    :=	 InvoiceRec.CHANGEDATE;
	OFAPLRec.LAST_UPDATE_LOGIN		    :=	 Null;
	OFAPLRec.CREATED_BY				    :=	 vENTERBYID;
	OFAPLRec.CREATION_DATE			    :=	 SYSDATE;
	OFAPLRec.ATTRIBUTE_CATEGORY		    :=	 Null;
	OFAPLRec.ATTRIBUTE1			        :=	 Null;
	OFAPLRec.ATTRIBUTE2			        :=	 Null;
	OFAPLRec.ATTRIBUTE3			        :=	 Null;
	OFAPLRec.ATTRIBUTE4			        :=	 Null;
	OFAPLRec.ATTRIBUTE5			        :=	 Null;
	OFAPLRec.ATTRIBUTE6			        :=	 Null;
	OFAPLRec.ATTRIBUTE7			        :=	 Null;
	OFAPLRec.ATTRIBUTE8			        :=	 Null;
	OFAPLRec.ATTRIBUTE9			        :=	 Null;
	OFAPLRec.ATTRIBUTE10			    :=	 Null;
	OFAPLRec.ATTRIBUTE11			    :=	 Null;
	OFAPLRec.ATTRIBUTE12			    :=	 Null;
	OFAPLRec.ATTRIBUTE13			    :=	 Null;
	OFAPLRec.ATTRIBUTE14			    :=	 Null;
	OFAPLRec.ATTRIBUTE15			    :=	 Null;
	OFAPLRec.ACCOUNT_SEGMENT     	    :=	 Null;
	OFAPLRec.BALANCING_SEGMENT   	    :=	 Null;
	OFAPLRec.COST_CENTER_SEGMENT 	    :=	 Null;
  	OFAPLRec.USSGL_TRANSACTION_CODE	    :=	 Null;
	OFAPLRec.STAT_AMOUNT			    :=	 Null;
	OFAPLRec.TYPE_1099                  :=	 Null;
	OFAPLRec.INCOME_TAX_REGION          :=	 Null;
	OFAPLRec.ASSETS_TRACKING_FLAG       :=	 Null;
	OFAPLRec.PRICE_CORRECTION_FLAG      :=	 Null;
	OFAPLRec.ORG_ID                     :=	 Null;
	OFAPLRec.RECEIPT_NUMBER             :=	 Null;
	OFAPLRec.RECEIPT_LINE_NUMBER        :=	 Null;
	OFAPLRec.MATCH_OPTION               :=	 Null;
	OFAPLRec.PACKING_SLIP               :=	 Null;
	OFAPLRec.RCV_TRANSACTION_ID         :=	 Null;
	OFAPLRec.PA_CC_AR_INVOICE_ID        :=	 Null;
	OFAPLRec.REFERENCE_1                :=	 Null;
	OFAPLRec.REFERENCE_2                :=	 Null;
	OFAPLRec.PA_CC_PROCESSED_CODE       :=	 Null;
	OFAPLRec.TAX_RECOVERY_RATE          :=	 Null;
	OFAPLRec.TAX_RECOVERY_OVERRIDE_FLAG :=	 Null;
	OFAPLRec.TAX_RECOVERABLE_FLAG       :=	 Null;
	OFAPLRec.TAX_CODE_OVERRIDE_FLAG     :=	 Null;

	/*
	*   Project Accounting information
	*/
    If InvoiceRec.OA_PROJECTID is not null Then
		OFAPLRec.PROJECT_ACCOUNTING_CONTEXT	 :=	'Y';
		OFAPLRec.PROJECT_ID					 :=	InvoiceRec.OA_PROJECTID;
		OFAPLRec.TASK_ID					 :=	InvoiceRec.OA_TASKID;
		OFAPLRec.EXPENDITURE_ITEM_DATE		 :=	SYSDATE;
		OFAPLRec.EXPENDITURE_TYPE			 :=	InvoiceRec.OA_EXPENDTYPE;
        OFAPLRec.EXPENDITURE_ORGANIZATION_ID := get_pa_orgid_sp(InvoiceRec.OA_CHARGE_ORG, InvoiceRec.SITEID);
		OFAPLRec.PA_ADDITION_FLAG		     :=	Null;
        OFAPLRec.PA_QUANTITY				 := InvoiceRec.INVOICEQTY;
	End If;

	/*
	*   Call user exit procedure
	*/
	MXE_USR_AP_SP(InvoiceRec, OFAPHRec, OFAPLRec, skip_transaction);

	--
	-- check for SKIP transaction
	--
	IF skip_transaction THEN
		RetProc(PrgName, 'User Exit Returned skip_transaction = true');
		Return;
	END IF;


	/*
	*  Determine if this invoice is already in the ap_expense table
	*  If yes then we're processing a multi-line invoice and we should
	*  only insert into the ap_expense_report_lines table (not the header)
	*/
    BEGIN

  	    vLine_only := TRUE;

        Select invoice_id
      	into OFAPHRec.INVOICE_ID
     	from ap_invoices_interface
     	where invoice_num = OFAPHRec.INVOICE_NUM
	    and vendor_id = OFAPHRec.VENDOR_ID;

	EXCEPTION
		When NO_DATA_FOUND then
  	        vLine_only := FALSE;
	   		select  ap_invoices_interface_s.nextval INTO OFAPHRec.INVOICE_ID from dual;

	END;

	OFAPLRec.INVOICE_ID		:=		OFAPHRec.INVOICE_ID;
	select  ap_invoice_lines_interface_s.nextval INTO OFAPLRec.INVOICE_LINE_ID from dual;
	
	/*
	Author: FG
	Date: 12-Dec-2007
	Description: If invoice does not have a PO registered, field UNIT_OF_MEAS_LOOKUP_CODE must be completed in order to show the UOM from the invoice line.
	This is a new field in open interface table for Oracle 12.
	*/
	If InvoiceRec.PONUM is null and InvoiceRec.INVOICEUNIT is not null then
		of_id_pkg.get_id_sp('UOM',
					InvoiceRec.INVOICEUNIT,
					InvoiceRec.SITEID,
					vORDERUNIT,
					Error_Status);
		If Error_Status != 0 then
			InvoiceRec.OA_IFACEMESSAGE := 'Invalid Unit of Measure:'||InvoiceRec.INVOICEUNIT;
			MaxApiErrors.RaiseApiError(PrgName, InvoiceRec.OA_IFACEMESSAGE);
		End If;
		
		OFAPLRec.UNIT_OF_MEAS_LOOKUP_CODE := InvoiceRec.INVOICEUNIT;
	End If;

	/*
	*   Insert into Oracle Financials AP Interface Table
	*/
    If NOT vLine_only then

		MaxDebugApi.Output('Insertion into AP_INVOICES_INTERFACE');
		MaxApiInsert.ins_rec(OFAPHRec);

	End If;

	MaxDebugApi.Output('Insertion into AP_INVOICE_LINES_INTERFACE');
	MaxApiInsert.ins_rec(OFAPLRec);

	/*
	*   End user routine
	*/
	RetProc(PrgName,ProcMsg);
EXCEPTION
	WHEN MaxApiErrors.ApiError
	OR MaxApiErrors.ApiWarning
	OR MaxApiErrors.ApiException THEN
		Raise;			 --- Re-raise if comes from lower
	WHEN OTHERS THEN
		MaxApiErrors.RaiseApiException(PrgName,ProcMsg);
END MXE_AP_SP;
/
SHOW ERRORS