CREATE OR REPLACE PROCEDURE API_INVCAPPR_SP
/*=========================================================================
* Licensed Materials - Property of IBM
*
* Restricted Materials of IBM
*
* 5724-M19
*
* (C) COPYRIGHT IBM CORP. 2008. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*==========================================================================
* Licensed Materials - Property of MRO Software
*
* Restricted Materials of MRO Software
*
* (C) COPYRIGHT MRO Software, Inc. 2001 - 2006. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with MRO Software, Inc.
*==========================================================================
*
* Filename:	OFAP.SP
* Procedure:	API_INVCAPPR_SP
*
* Description:
*
*	Procedure to pass data coming from Oracle Financials to Maximo.
*	API that is called from ORACLE'S Invoice
*
* Public Functions:
*
*
* Modification History
*
*	05-Apr-1996	Pradeep Kumar	Created.
*	24-feb-1997	Raja Krishnasamy	Changed code to check for MAXAPREC.ITEMNUM - If MAXITEM THEN REPLACE WITH NULL
*   19-may-1997 Guido Viarisio  check for null for att14
*   15-aug-2000 Tom Sarasin     changed code to create a MAXIMO CRT MEMO from an Oracle Invoice Cancellation
*	23-Jan-2001	Kalpana Manne	Upgrade to 4.1.1
*								added column : einvoice
*   04-Jan-2002	Kalpana Manne   Upgrade to 5.1
*								Added the columns ORGID, SITEID
*                               Moved the Maximo datalookups and control values to Java ERP exit
*	14-FEB-2002	Kalpana Manne   Populate same transid for all records of an invoice
* 	15-May-2002	Kalpana Manne   To get orgid for single operating unit with no client info
* 	15-Nov-2002	Kalpana Manne   Passing invc_client_info to the userdata4 column
*   06-Mar-2003 Kalpana Manne	Renamed WONUM to REFWO.
*   10-Mar-2003 Kalpana Manne	Made the HISTORYFLAG to null.
*   26-Mar-2003 Kalpana Manne	Mapped the Oracle Invoice types to Maximo Document types.
*   04-Apr-2003 Kalpana Manne   Removed UPPER for PONUM's and INVOICENUM's to support ALN Ponum's
*                               and ALN Invoicenum's for issue 032106.
*   02-Jul-2003	Kalpana Manne	Issue 034075 - instead of header level ponum, using
*                               line level invoicelineponum to support multiple PO matching.
*   21-Aug-2003 AKJ             Issue_025468 - If line is from Maximo PO then
*                               use Maximo PO and PO Line Numbers.
*   24-Aug-2004 AKJ				Issue_68591 - Don't set polinenum if no ponum.
*	05-Jan-2005 KM  			Upgrade to Release 6.0
*   24-Oct-2007 KM  			Upgraded to 7.1 release
*   12-Jun-2008 AKJ             Issue_08-16928 - Set the PO Line Num correctly for PO's not from Maximo.
*   17-Jun-2008 KM              Issue_08-16943 For Amount based PO line types
*								set INVOICEQTY, INVOICEUNITCOST and QUANTITY to null.
*==========================================================================*/
(
	APINVCREC		    IN AP_INVOICES_ALL%ROWTYPE,
	APDISTREC		    IN AP_INVOICE_DISTRIBUTIONS_ALL%ROWTYPE,
	error_status		IN OUT NUMBER  	-- oracle financials error code
) IS

	-- Issue_08-23591 Author: FG Date: Oct-30-2008 add DOCUMENT_CREATION_METHOD and NOTE_TO_RECEIVER to the query
	CURSOR  po_dis(podis_id PO_DISTRIBUTIONS.PO_DISTRIBUTION_ID%TYPE) IS
    SELECT  ph.SEGMENT1, ph.INTERFACE_SOURCE_CODE, ph.SHIP_VIA_LOOKUP_CODE,
  	  	ph.FOB_LOOKUP_CODE, ph.FREIGHT_TERMS_LOOKUP_CODE,
		ph.DOCUMENT_CREATION_METHOD, ph.NOTE_TO_RECEIVER, 
  	  	pl.LINE_NUM, pl.ITEM_DESCRIPTION,
  	  	ps.SHIPMENT_NUM, pd.DISTRIBUTION_NUM,
 	  	pd.QUANTITY_ORDERED, pd.QUANTITY_DELIVERED, pd.QUANTITY_BILLED,
  	  	pd.QUANTITY_CANCELLED, pd.AMOUNT_BILLED, pd.REQ_DISTRIBUTION_ID,
	  	pd.PO_RELEASE_ID, pd.ATTRIBUTE1, pd.ATTRIBUTE2, plt.ORDER_TYPE_LOOKUP_CODE
  	FROM    po_headers ph, po_lines pl, po_line_locations ps, po_distributions pd, po_line_types plt
  	WHERE   pd.PO_DISTRIBUTION_ID = podis_id
  	AND	  ps.LINE_LOCATION_ID = pd.LINE_LOCATION_ID
  	AND	  pl.PO_LINE_ID = pd.PO_LINE_ID
  	AND	  ph.PO_HEADER_ID = pd.PO_HEADER_ID
  	AND	  pl.LINE_TYPE_ID = plt.LINE_TYPE_ID;

	PrgName			 VARCHAR2(40) := 'API_INVCAPPR_SP';
	ProcMsg			 VARCHAR2(100);
	error_msg		 VARCHAR2(50);
	error_text		 VARCHAR2(2000);
	MAXAPREC		 MXINVOICE_IFACE%ROWTYPE;

	invc_client_info    ap_invoices_all.ORG_ID%TYPE;

	invc_ap_ccid      	ap_invoices_all.ACCTS_PAY_CODE_COMBINATION_ID%Type;

	invc_po_no		    po_headers.segment1%Type;
	invc_po_source      po_headers.interface_source_code%Type;
	invc_reqdis_id		po_distributions.req_distribution_id%Type;
	invc_porel_id		po_distributions.po_release_id%Type;
	invc_porel_num		po_releases.release_num%Type;
	invc_porel_at15	    po_releases.attribute15%Type;
	invc_shipvia		po_headers.ship_via_lookup_code%Type;
	invc_fob		    po_headers.fob_lookup_code%Type;
	invc_freight		po_headers.freight_terms_lookup_code%Type;
	invc_pl_no		    po_lines.line_num%Type;
	invc_itm_desc		po_lines.item_description%Type;
	invc_ps_no		    po_line_locations.shipment_num%Type;
	invc_pd_no		    po_distributions.distribution_num%Type;
	invc_qty_ord		po_distributions.quantity_ordered%Type;
	invc_qty_rcv		po_distributions.quantity_delivered%Type;
	invc_qty_invc		po_distributions.quantity_billed%Type;
	invc_qty_can		po_distributions.quantity_cancelled%Type;
	invc_amt_invc		po_distributions.amount_billed%Type;
	invc_ps_pono		po_distributions.attribute1%Type;
	invc_ps_polineno	po_distributions.attribute2%Type;
	invc_plt_ordtyp		po_line_types.order_type_lookup_code%Type;
    invc_vnd_no       	ap_suppliers.SEGMENT1%Type;
    invc_vndsite_code 	ap_supplier_sites_all.VENDOR_SITE_CODE%Type;

	vhold           	NUMBER;
	skip_transaction 	BOOLEAN;

	-- Author: FG Date: Oct-31-2008 new variables with values in version 5.X - 6.X 
	invc_po_creation_method	po_headers.DOCUMENT_CREATION_METHOD%Type;
	invc_po_note	     po_headers.NOTE_TO_RECEIVER%Type;	

BEGIN
	ProcMsg := 'ORAFIN Invoice API process for: '||APINVCREC.invoice_num;
	EnterProc(PrgName,ProcMsg);
/*
*	Check if error in trigger
*/
	IF error_status IS NOT NULL AND (error_status < 0 or error_status = 100) THEN
		error_text   := SQLERRM(error_status);
		error_msg    := ('Error from ORAFIN Invoice trigger: ');
		MaxApiErrors.ApiErrorInsert(PrgName,error_msg,error_text);
            RetProc(PrgName,'Returned because of previous error');
		Return;
	END IF;

	invc_client_info := NVL(APDISTREC.ORG_ID, APINVCREC.ORG_ID);
	MaxDebugApi.Output('invc_client_info: '||invc_client_info);
    MAXAPREC.SITEID := RTRIM(SUBSTR(invc_client_info,0,8));
	MaxDebugApi.Output('MAXAPREC.SITEID: '||MAXAPREC.SITEID);

    -- 04-Jan-2002 Getting the ledger_id which is orgid in Maximo
    BEGIN
	   IF invc_client_info IS NOT NULL THEN
	   --Muliple operating unit or Single operating unit with client_info
	     SELECT	  set_of_books_id
	     INTO		MAXAPREC.ORGID
	     FROM		hr_operating_units
	     WHERE		ORGANIZATION_ID = invc_client_info;
	   ELSE
	   --15-May-2002 Single operating unit where client_info is null
	     SELECT	  ledger_id
	     INTO	  MAXAPREC.ORGID
	     FROM	  gl_ledgers;
	   END IF;

	EXCEPTION
	   When others then
	    error_msg	:=	'Error from gl_ledgers..';
		error_text	:=	RetProcError(PrgName,error_msg);
		return;
	END;

	MAXAPREC.INVOICENUM		:=  APINVCREC.INVOICE_NUM;
	MAXAPREC.INVOICEDESC	:=	APINVCREC.DESCRIPTION;
	MAXAPREC.EXTERNALREFID	:=	APINVCREC.INVOICE_ID;

	IF APDISTREC.PO_DISTRIBUTION_ID IS NOT NULL THEN

		Put_In_Log(PrgName,'Getting po distribution...');

		OPEN po_dis(APDISTREC.PO_DISTRIBUTION_ID);

		-- Issue_08-23591 Author: FG Date: Oct-30-2008 add invc_po_creation_method and invc_po_note
		FETCH po_dis INTO invc_po_no, invc_po_source, invc_shipvia,
		invc_fob, invc_freight, 
		invc_po_creation_method, invc_po_note, 
		invc_pl_no, invc_itm_desc,
		invc_ps_no, invc_pd_no,
		invc_qty_ord, invc_qty_rcv, invc_qty_invc,
		invc_qty_can, invc_amt_invc, invc_reqdis_id, invc_porel_id,
		invc_ps_pono, invc_ps_polineno, invc_plt_ordtyp;

		IF po_dis%NOTFOUND THEN
			error_status := 100;
		END IF;

		IF po_dis%ISOPEN THEN
			CLOSE po_dis;
		END IF;
	END IF;

	invc_ap_ccid := NVL(APDISTREC.ACCTS_PAY_CODE_COMBINATION_ID,APINVCREC.ACCTS_PAY_CODE_COMBINATION_ID);

	IF invc_ap_ccid IS NOT NULL THEN
		api_glcc_pkg.gl_xxccid_sp(invc_ap_ccid,
		                          MAXAPREC.GLCREDITACCT,
		                          error_status);
	END IF;

	IF APDISTREC.DIST_CODE_COMBINATION_ID IS NOT NULL THEN
		api_glcc_pkg.gl_xxccid_sp(APDISTREC.DIST_CODE_COMBINATION_ID,
		                          MAXAPREC.GLDEBITACCT,
		                          error_status);
	END IF;

	IF invc_porel_id IS NOT NULL THEN
	  	BEGIN
			SELECT  release_num, attribute15
			INTO	invc_porel_num, invc_porel_at15
			FROM	po_releases p
			WHERE	p.po_release_id = invc_porel_id;
		EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in PO Release...');
		END;
	END IF;

	IF APINVCREC.VENDOR_ID IS NOT NULL THEN
	  	BEGIN
			SELECT	v.segment1
			INTO	invc_vnd_no
			FROM	ap_suppliers v
			WHERE	v.vendor_id = APINVCREC.VENDOR_ID;
		EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in supplier...');
		END;
	END IF;

	IF APINVCREC.VENDOR_SITE_ID IS NOT NULL THEN
	  	BEGIN
			SELECT	s.vendor_site_code
			INTO	invc_vndsite_code
			FROM	ap_supplier_sites_all s
			WHERE	s.vendor_site_id = APINVCREC.VENDOR_SITE_ID;
		EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in vendor site...');
		END;
	END IF;

	IF APINVCREC.TERMS_ID IS NOT NULL THEN
	  	BEGIN
			SELECT	name
			INTO	MAXAPREC.PAYMENTTERMS
			FROM	ap_terms
			WHERE	term_id = APINVCREC.TERMS_ID;
		EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in term_id...');
		END;
	END IF;

	Put_In_Log(PrgName,'user'||APINVCREC.CREATED_BY);
	IF APINVCREC.CREATED_BY IS NOT NULL THEN
	  	BEGIN
			SELECT	user_name
			INTO	MAXAPREC.ENTERBY
			FROM	fnd_user
			WHERE	user_id = APINVCREC.CREATED_BY;
		EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in created_by...');
		END;
	END IF;

    If invc_porel_num Is Not Null then
	   -- this is a blanket release
	   if invc_porel_at15 like 'MAXIMO:%' then		-- originated in MAXIMO
	   	  MAXAPREC.INVOICELINEPONUM := substr(invc_porel_at15,8,length(invc_porel_at15));
	      --MAXAPREC.INVOICELINEPONUM	:= invc_ps_pono;
	      --rel_polinenum := invc_ps_polineno;
	   else											-- originated in Oracle
	      MAXAPREC.INVOICELINEPONUM := invc_po_no||'-'||invc_porel_num;
	   end if;
	else
	   -- THIS IS A STD PO
	   MAXAPREC.INVOICELINEPONUM   := 	invc_po_no;
	end if;

	IF trim(invc_client_info) IS NOT NULL THEN
      MAXAPREC.VENDOR		:=		UPPER(trim(invc_client_info))||'-'||UPPER(invc_vnd_no)||'-'||UPPER(invc_vndsite_code);
	ELSE
	  MAXAPREC.VENDOR		:=		'0'||'-'||UPPER(invc_vnd_no)||'-'||UPPER(invc_vndsite_code);
	END IF;

	MAXAPREC.CONTACT		:=		Null;
	MAXAPREC.CHECKCODE		:=		Null;
	MAXAPREC.CHECKNUM		:=		Null;
	MAXAPREC.CURRENCYCODE	:=		UPPER(APINVCREC.invoice_currency_code);
	MAXAPREC.EXCHANGERATE	:=		APDISTREC.exchange_rate;
	MAXAPREC.EXCHANGEDATE	:=		APDISTREC.exchange_date;
    MAXAPREC.EXCHANGERATE2	    :=  MaxUtlPack.GetExchangeRate(APDISTREC.mrc_exchange_rate);
	MAXAPREC.DOCUMENTTYPE	  	:=	APINVCREC.invoice_type_lookup_code;
	MAXAPREC.ORIGINVOICENUM	  	:=	Null;
	IF NVL(APDISTREC.POSTED_FLAG,'*') = 'Y' and NVL(APINVCREC.PAYMENT_STATUS_FLAG,'*') = 'Y' THEN
		MAXAPREC.STATUS	:=	'PAID';
	ELSE
		MAXAPREC.STATUS	:=	'APPR';
	END IF;
	MAXAPREC.STATUSDATE		  	:=	APDISTREC.creation_date;
	MAXAPREC.APPROVALNUM	  	:=	Null;
	MAXAPREC.INVOICEDATE	  	:=	APINVCREC.invoice_date;
	MAXAPREC.DISCOUNTDATE	  	:=	Null;
	MAXAPREC.DUEDATE		  	:=	Null;
	MAXAPREC.PAIDDATE		  	:=	Null;
	MAXAPREC.PAID			  	:=	APINVCREC.amount_paid;
	MAXAPREC.ENTERDATE		  	:=	NVL(APINVCREC.creation_date,SYSDATE);
	MAXAPREC.CHANGEDATE		  	:=	NVL(APDISTREC.creation_date,SYSDATE);
	MAXAPREC.CHANGEBY		  	:=	MAXAPREC.ENTERBY;
	MAXAPREC.IVOL_ENTERBY	  	:=	MAXAPREC.ENTERBY;
	MAXAPREC.HISTORYFLAG	  	:=	Null;
	MAXAPREC.TOTALTAX1		  	:=	Null;
	MAXAPREC.TAX1GL			  	:=	Null;
	MAXAPREC.TOTALTAX2		  	:=	Null;
	MAXAPREC.TAX2GL			  	:=	Null;
	MAXAPREC.TOTALTAX3		  	:=	Null;
	MAXAPREC.TAX3GL			  	:=	Null;
	MAXAPREC.TOTALTAX4		  	:=	Null;
	MAXAPREC.TAX4GL			  	:=	Null;
	MAXAPREC.INCLUSIVE1		  	:=	Null;
	MAXAPREC.INCLUSIVE2		  	:=	Null;
	MAXAPREC.INCLUSIVE3		  	:=	Null;
	MAXAPREC.DISCOUNT		  	:=	APINVCREC.discount_amount_taken;
	MAXAPREC.TOTALCOST		  	:=	APINVCREC.invoice_amount;
	MAXAPREC.TOTALTAX5		  	:=	Null;
	MAXAPREC.TAX5GL			  	:=	Null;
	MAXAPREC.INCLUSIVE4		  	:=	Null;
	MAXAPREC.INCLUSIVE5		  	:=	Null;
	MAXAPREC.VENDORINVOICENUM 	:= 	MAXAPREC.INVOICENUM;
	MAXAPREC.BASETOTALCOST 	  	:=	Null;
	MAXAPREC.APSUSPENSEACCT	  	:=	Null;
	MAXAPREC.APCONTROLACCT 	  	:=	Null;
	MAXAPREC.FINANCIALPERIOD  	:=	APDISTREC.period_name;
	MAXAPREC.INVOICELINENUM	  	:=	'1'||LTRIM(TO_CHAR(APDISTREC.INVOICE_LINE_NUMBER,'0999'))||LTRIM(TO_CHAR(APDISTREC.DISTRIBUTION_LINE_NUMBER,'0999'));

--	MAXAPREC.INVOICELINENUM	  	:=	APDISTREC.distribution_line_number;
	MAXAPREC.PONUM 			  	:=	Null;

    if MAXAPREC.INVOICELINEPONUM is not null then
		-- Author: FG Date: 28-Oct-2008 Include rcv_creation_method check
		If invc_po_source is null Then
			If (upper(invc_po_note) like upper('Approved MAXIMO Purchase Order%') and  nvl(invc_po_creation_method,'*') <> 'COPY_DOCUMENT') Then
				invc_po_source := 'MAXIMO';
			End If;
		End If;

        if invc_ps_polineno is not null and invc_po_source = 'MAXIMO' then
           MAXAPREC.POLINENUM := invc_ps_polineno;
        else
           MAXAPREC.POLINENUM := '1'||LTRIM(TO_CHAR(invc_pl_no,'0999'))||LTRIM(TO_CHAR(invc_ps_no,'0999'))||LTRIM(TO_CHAR(invc_pd_no,'0999'));
        end if;
    end if;

	IF invc_plt_ordtyp = 'AMOUNT' THEN
		MAXAPREC.INVOICEQTY		    :=	 Null;
		MAXAPREC.INVOICEUNITCOST	:=	 Null;
		MAXAPREC.QUANTITY		    :=	 Null;
	ELSE
		MAXAPREC.INVOICEQTY		    :=	APDISTREC.quantity_invoiced;
		MAXAPREC.INVOICEUNITCOST	:=	APDISTREC.unit_price;
		MAXAPREC.QUANTITY		    :=	APDISTREC.quantity_invoiced;
    END IF;	 

	MAXAPREC.DESCRIPTION		:=	NVL(APDISTREC.description,invc_itm_desc);
	MAXAPREC.INVOICEUNIT		:=	Null;
	MAXAPREC.INVOICELINECOST	:=	NVL(APDISTREC.base_amount,APDISTREC.amount);
	MAXAPREC.CONVERSION		    :=	Null;
	MAXAPREC.RECEIPTREQD	    :=	Null;
	MAXAPREC.TAX1CODE		    :=	Null;
	MAXAPREC.TAX1			    :=	Null;
	MAXAPREC.TAX2CODE		    :=	Null;
	MAXAPREC.TAX2			    :=	Null;
	MAXAPREC.TAX3CODE		    :=	Null;
	MAXAPREC.TAX3			    :=	Null;
	MAXAPREC.INVOICELIN1	    :=	Null;
	MAXAPREC.INVOICELIN2	    :=	Null;
	MAXAPREC.INVOICELIN3	    :=	Null;
	MAXAPREC.INVOICELIN4	    :=	Null;
	MAXAPREC.INVOICELIN5	    :=	Null;
	MAXAPREC.TAX4CODE		    :=	Null;
	MAXAPREC.TAX4			    :=	Null;
	MAXAPREC.TAX5CODE		    :=	Null;
	MAXAPREC.TAX5			    :=	Null;
	MAXAPREC.LOADEDCOST		    :=	NVL(MAXAPREC.INVOICELINECOST,0);
	MAXAPREC.PRORATED		    :=	0;
	MAXAPREC.PRORATECOST	    :=	Null;
	MAXAPREC.PRORATESERVICE	    :=	0;
	MAXAPREC.LINECOST2		    :=	Null;
	MAXAPREC.REFWO			    :=	Null;
	MAXAPREC.LINECOST		    :=	APDISTREC.amount;
	MAXAPREC.PERCENTAGE		    :=	Null;
	MAXAPREC.MEMO			    :=	APDISTREC.line_type_lookup_code;
	MAXAPREC.OA_APLINETYPE		:=	APDISTREC.line_type_lookup_code;
	MAXAPREC.COSTLINENUM	    :=	APDISTREC.distribution_line_number;
	MAXAPREC.CHARGESTORE	    :=	Null;
	MAXAPREC.ICT1			    :=	Null;
	MAXAPREC.ICT2			    :=	Null;
	MAXAPREC.ICT3			    :=	Null;
	MAXAPREC.LOCATION		    :=	Null;
	MAXAPREC.BANKACCOUNT	    :=	Null;
	MAXAPREC.PRICEVAR		    :=	NVL(APDISTREC.base_invoice_price_variance,0);
	MAXAPREC.OWNERSYSID 	  	:= 'OA';
	MAXAPREC.SENDERSYSID        :=  Null;
	MAXAPREC.TRANSSEQ 		 	:=	APDISTREC.DISTRIBUTION_LINE_NUMBER;
	MAXAPREC.OA_IFACESTATUS		:=  'DONE';
	MAXAPREC.OA_IFACETIMESTAMP	:= 	SYSDATE;

/*
*   15-aug-2000
*   new logic for Invoice Cancellation
*    make credit memo invoice to accomodate invoice cancel
*/
    If APDISTREC.reversal_flag = 'Y' then   -- (this is a Cancelled Invoice)
	   MAXAPREC.DOCUMENTTYPE := 'CREDIT';

	   MAXAPREC.TOTALCOST := NULL; -- setting totalcost to null will force the
								   --  MIG to calculate based on lines on invoice
	   MAXAPREC.INVOICENUM := MAXAPREC.INVOICENUM||'CR';
	   MAXAPREC.VENDORINVOICENUM   := MAXAPREC.INVOICENUM;
    End If;

/*
*   Call user exit procedure
*/
	MOF_USR_AP_SP(APINVCREC, APDISTREC, MAXAPREC, skip_transaction);

	IF skip_transaction THEN
		RetProc(PrgName,'User Exit Returned skip_transaction = true');
		Return;
	END IF;

/*
*	Insert into MXIN_INTER_TRANS table prior to interface table
*/
	Begin
	 select transid
	 into   MAXAPREC.transid
	 from   mxinvoice_iface
	 where  invoicenum=MAXAPREC.INVOICENUM
	 and    rownum<2;
	Exception
	 When NO_DATA_FOUND THEN
	     SELECT maxifacetransseq.nextval
	     INTO   MAXAPREC.transid
	     FROM DUAL;
	END;

	select count(*)
	    into vhold
	from mxin_inter_trans mit
	where mit.transid = MAXAPREC.transid;

	If vhold = 0 then
	   MaxUtlPack.PutApiTransIn(MaxutlPack.GetMaxOraCtl('EXTSYS'),
				 'MXINVOICE_FROA12',
				 MAXAPREC.TRANSID,
				 null);
	End If;

/*
* 	Insert into MAXIMO Invoice Interface table
*/
	MaxApiInsert.ins_rec(MAXAPREC);


 	RetProc(PrgName,ProcMsg);

EXCEPTION
	WHEN MaxApiErrors.ApiError
	OR MaxApiErrors.ApiWarning
	OR MaxApiErrors.ApiException THEN
		IF MaxUtlPack.IsMaxOraCtlYES('EXCEPTIONTOEXT') THEN
			Raise;			-- re-raise API errors
		ELSE
			error_text := RetProcError(PrgName,error_text);
		END IF;
	WHEN OTHERS THEN
		IF MaxUtlPack.IsMaxOraCtlYES('EXCEPTIONTOEXT') THEN
			MaxApiErrors.RaiseApiException(PrgName,ProcMsg);
		ELSE
			error_text := RetProcError(PrgName,error_text);
		END IF;
END  API_INVCAPPR_SP;
/
SHOW ERRORS

