CREATE OR REPLACE PROCEDURE API_PC_SP
/*=========================================================================
* Licensed Materials - Property of IBM
*
* Restricted Materials of IBM
*
* 5724-M19
*
* (C) COPYRIGHT IBM CORP. 2008. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*==========================================================================
* Licensed Materials - Property of MRO Software
*
* Restricted Materials of MRO Software
*
* (C) COPYRIGHT MRO Software, Inc. 2001 - 2006. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with MRO Software, Inc.
*==========================================================================
*
* Filename:	OFPC.SP
* Procedure:	API_PC_SP
*
* Description:
*
*	Procedure to pass Blanket/Planned Purchase Orders
*		from Oracle Financials to Maximo.
*	API that is called from ORACLE'S PO approval
*
* Public Functions:
*                API_PC_SP
*
* Modification History
*
*	31-Jan-2005	Kalpana Manne	Created
*   2-Oct-2007 Igor Stockler Upgraded to Release 7.1
*                                  -Updated IFACENAME ID;
*                                  -Added 'FIXED PRICE' and 'RATE' to lines types translated 
*                                    into 'SERVICE' type .
*   30-Oct-2007 Igor Stockler Update 'SERVICE' Line Type according to tentative changes
*                                             to the spec. Amount will be mapped according to AMOUNT column
*                                             in the PO_LINE.
*   20-Nov-2007 Igor Stockler Updated 'SERVICE' line type mapping according to rules
*                                             defined in the spec addendum. related issue # 07-14558
*   03-Sep-2008 Fabiana Rocha Fix for Issue#08-20792 - Send the previous revision num to Maximo when cancelling 
*  										a contract in Oracle Applications.
*   24-Nov-2008 Fabiana Rocha Undo change for Issue#08-20792 and add piece of code to send revision comments when 
* 										re-approving a purchase contract in Oracle Applications
*==========================================================================*/
(
	POACTREC    IN po_action_history%RowType,
	POHEADREC   IN po_headers_all%RowType,
	OFPOAPIRec	IN api_poact_pkg.PORec
) IS
	PrgName			VARCHAR2(40) := 'API_PC_SP';
	ProcMsg			VARCHAR2(100);
	error_msg		VARCHAR2(50);
	error_text		VARCHAR2(2000);
	MXPCREC		 	mxpc_iface%ROWTYPE;
	MXPCLINEREC		mxpc_iface%ROWTYPE;

  	skip_transaction BOOLEAN;
	skip_line        BOOLEAN;
	v_transseq  	 NUMBER :=0;
	error_Status     NUMBER;
/*
CURSOR po_ln(ph_id PO_HEADERS.PO_HEADER_ID%TYPE) IS
SELECT pl.last_update_date, pl.last_updated_by, pl.line_num, pl.item_id,
       pl.item_description, pl.unit_meas_lookup_code, pl.quantity,
	   pl.unit_price, pl.reference_num,	pl.vendor_product_num, pl.contract_num,
	   plt.line_type, plt.order_type_lookup_code, pl.capital_expense_flag,
	   pl.cancel_flag, pl.closed_code, pl.closed_flag,
	   pl.attribute1, pl.attribute2, pl.attribute3, pl.attribute4, pl.attribute5
  FROM po_lines pl,
       po_line_types plt
 WHERE pl.po_header_id = ph_id
   AND pl.line_type_id = plt.line_type_id(+);
*/
CURSOR po_ln(ph_id PO_HEADERS.PO_HEADER_ID%TYPE) IS
SELECT pl.last_update_date, pl.last_updated_by, pl.line_num, pl.item_id,
       pl.item_description, pl.unit_meas_lookup_code, pl.quantity pl_qty,
       ps.shipment_num, ps.quantity ps_qty, ps.quantity_received,
       ps.quantity_accepted, ps.quantity_rejected,
       ps.quantity_billed ps_billqty, ps.quantity_cancelled ps_canqty,
       ps.ship_to_location_id, ps.ship_to_organization_id, pl.unit_price,
       pl.reference_num, ps.firm_date, ps.promised_date,
       pl.vendor_product_num, pl.contract_num, ps.taxable_flag,
       pl.cancel_flag, pl.closed_code, pl.closed_flag,
       ps.tax_code_id, ps.estimated_tax_amount, ps.need_by_date,
       plt.line_type, plt.order_type_lookup_code, pl.capital_expense_flag,
       pd.distribution_num, pd.accrual_account_id, pd.destination_organization_id,
       pd.destination_subinventory, pd.destination_type_code,
       pd.quantity_ordered, pd.quantity_delivered,
       pd.quantity_billed pd_billqty, pd.quantity_cancelled pd_canqty,
       pd.amount_billed, pd.code_combination_id, pd.req_distribution_id,
       pd.po_distribution_id, pl.attribute1, pl.attribute2, pl.attribute3,
       pl.attribute4, pl.attribute5, ps.price_override, ps.receiving_routing_id,
       pd.mrc_rate, pl.amount
  FROM po_lines pl,
       po_line_locations ps,
       po_distributions pd,
       po_line_types plt
 WHERE pl.po_header_id = ph_id
   AND pl.line_type_id = plt.line_type_id(+)
   AND ps.po_line_id(+) = pl.po_line_id
   AND pd.line_location_id(+) = ps.line_location_id
   AND pd.po_release_id(+) IS NULL
   AND ps.po_release_id(+) IS NULL;

BEGIN
	ProcMsg := 'ORAFIN PO API process for: '||OFPOAPIRec.po_no||' po_type:'||OFPOAPIRec.po_type;
	EnterProc(PrgName,ProcMsg);
/*
*	Check if error in trigger
*/
	IF OFPOAPIRec.error_status IS NOT NULL AND (OFPOAPIRec.error_status < 0 or OFPOAPIRec.error_status = 100) THEN
		error_text   := SQLERRM(OFPOAPIRec.error_status);
		error_msg    := 'Error from ORAFIN PO trigger: ';
        MaxApiErrors.ApiErrorInsert(PrgName,error_msg,error_text);
		RetProc(PrgName,ProcMsg);
		Return;
	END IF;

	MaxDebugApi.Log(PrgName,'Output 0');

    MaxDebugApi.Output('OFPOAPIRec.client_info: '||OFPOAPIRec.client_info);
    MXPCREC.AUTHSITEID := RTRIM(SUBSTR(OFPOAPIRec.client_info,0,8));
	MaxDebugApi.Output('MXPCREC.AUTHSITEID: '||MXPCREC.AUTHSITEID);

    -- 04-Jan-2002 Getting the set_of_books_id which is orgid in Maximo
    BEGIN
	   IF OFPOAPIRec.client_info IS NOT NULL THEN
	   --Muliple operating unit or Single operating unit with client_info
	     SELECT	  set_of_books_id
	     INTO		MXPCREC.ORGID
	     FROM		hr_operating_units
	     WHERE		ORGANIZATION_ID = OFPOAPIRec.client_info;
	   ELSE
	   --15-May-2002 Single operating unit where client_info is null
	     SELECT	  ledger_id
	     INTO		MXPCREC.ORGID
	     FROM		gl_ledgers;
	   END IF;
	EXCEPTION
	   When others then
	    error_msg	:=	'Error from gl_sets_of_books ';
		error_text	:=	RetProcError(PrgName,error_msg);
	  	return;
	END;
	MaxDebugApi.Output('set_of_books_id: '||MXPCREC.ORGID);
	-- 04-Jan-2002

	MXPCREC.AUTHORGID	   	:=	    MXPCREC.ORGID;
	MXPCREC.CONTRACTNUM		:=		OFPOAPIRec.po_no;
	MXPCREC.DESCRIPTION		:=		NVL(OFPOAPIRec.po_comments,'Purchase Order from Oracle Financials');
    MXPCREC.PURCHASEAGENT	:=		OFPOAPIRec.po_buyer;
	MaxDebugApi.Log('MXPCREC.PURCHASEAGENT: ',MXPCREC.PURCHASEAGENT);

 	MXPCREC.ENTERDATE		:=		OFPOAPIRec.po_enter_date;

	MXPCREC.CONTRACTTYPE	:=   	OFPOAPIRec.po_type;
	MXPCREC.REVISIONNUM		:=	 	POHEADREC.REVISION_NUM;

	MaxDebugApi.Log('POHEADREC.REVISION_NUM: ',POHEADREC.REVISION_NUM);
	MaxDebugApi.Log('MXPCREC.CONTRACTTYPE: ',MXPCREC.CONTRACTTYPE);

	MaxDebugApi.Log(PrgName,'Output 00');

	IF trim(OFPOAPIRec.client_info) IS NOT NULL THEN
	  MXPCREC.VENDOR		:=		trim(OFPOAPIRec.client_info)||'-'||OFPOAPIRec.po_vnd_no||'-'||OFPOAPIRec.po_vndsite_no;
	ELSE
	  MXPCREC.VENDOR		:=		'0'||'-'||OFPOAPIRec.po_vnd_no||'-'||OFPOAPIRec.po_vndsite_no;
	END IF;
	MXPCREC.CONTACT			:=		OFPOAPIRec.po_vndctc_no;
	MXPCREC.FREIGHTTERMS	:=		OFPOAPIRec.po_freight;
	MXPCREC.PAYMENTTERMS	:=		OFPOAPIRec.po_terms;
	MXPCREC.SHIPVIA			:=		OFPOAPIRec.po_shipvia;
	MXPCREC.CUSTOMERNUM		:=		OFPOAPIRec.po_vndord_no;
	MXPCREC.FOB				:=		OFPOAPIRec.po_fob;
	MXPCREC.BILLTO			:=		OFPOAPIRec.po_billto;
	MXPCREC.BILLTOATTN		:=		Null;

	MXPCREC.TOTALCOST		:=		null;
	MXPCREC.MAXVOL			:= 		NVL(POHEADREC.AMOUNT_LIMIT, OFPOAPIRec.po_blanket_amount);
	IF 	MXPCREC.CONTRACTTYPE = 'BLANKET' THEN
		MXPCREC.CANEXCEEDVOLUME := 0;
	END IF;
	MXPCREC.MAXRELVOL		:= 		null;
	MXPCREC.CHANGEBY		:=		OFPOAPIRec.po_change_by;
	MXPCREC.CHANGEDATE		:=		OFPOAPIRec.po_change_date;

    MXPCREC.HISTORYFLAG		:=		null;
	MXPCREC.CURRENCYCODE	:=		UPPER(OFPOAPIRec.po_currency);
	MXPCREC.EXCHANGERATE	:=		nvl(OFPOAPIRec.po_xchg,1);
	MXPCREC.EXCHANGEDATE	:=		OFPOAPIRec.po_xchg_date;

	MaxDebugApi.Log(PrgName,'Output 2');

	MXPCREC.STATUSDATE		:=		OFPOAPIRec.po_change_date;
	MXPCREC.STARTDATE		:=		POHEADREC.START_DATE_ACTIVE;
	MXPCREC.ENDDATE			:=		POHEADREC.END_DATE_ACTIVE;

    MaxDebugApi.Log(PrgName,'po_close:'||OFPOAPIRec.po_close||' po_cancel:'||OFPOAPIRec.po_cancel
                            ||' po_approved:'||OFPOAPIRec.po_approved);

	IF NVL(OFPOAPIRec.po_cancel,'*') = 'Y' THEN
		MXPCREC.STATUSIFACE	:=	1;
		MXPCREC.STATUS		:=	'CAN';

	ELSIF NVL(OFPOAPIRec.po_close,'*') = 'CLOSED' OR NVL(OFPOAPIRec.po_close,'*') = 'FINALLY CLOSED' THEN
		MXPCREC.STATUSIFACE	:=	1;
		MXPCREC.STATUS		:=	'CLOSE';
	ELSIF NVL(OFPOAPIRec.po_approved,'*') = 'R' AND POHEADREC.REVISION_NUM	!= 0 THEN
		MXPCREC.STATUSIFACE	:=	0;
		MXPCREC.STATUS		:=	Null;
		MXPCREC.REVCOMMENTS := 'oa_docrevised'; -- Document revised in Oracle Applications.
	ELSIF NVL(OFPOAPIRec.po_approved,'Y') = 'Y' THEN
		MXPCREC.STATUSIFACE	:=	0;
		MXPCREC.STATUS		:=	'APPR';

		/**
		 * Fix for Issue#08-20792
		 * Send revision comments when re-approving a purchase contract, a new revsion was created in Oracle Applications 
		 */
		if (MXPCREC.REVISIONNUM <> 0) then
			MXPCREC.REVCOMMENTS := 'oa_docrevised'; -- Document revised in Oracle Applications.
		end if;
    END IF;

    MaxDebugApi.Log(PrgName,'MXPCREC.STATUSIFACE:'||MXPCREC.STATUSIFACE);
    --
    -- Call user exit procedure for the header
    --
	MOF_USR_PC_SP(POACTREC, POHEADREC, OFPOAPIRec, MXPCREC, skip_line, skip_transaction);

  	IF skip_transaction THEN
		RetProc(PrgName,'User Exit Returned skip_transaction = true');
		Return;
	END IF;

	SELECT maxifacetransseq.nextval INTO MXPCREC.TRANSID FROM DUAL;

 	IF NVL(MXPCREC.STATUSIFACE,'0') = '1' OR MXPCREC.CONTRACTTYPE = 'CONTRACT' THEN
		MXPCREC.OA_IFACESTATUS	:= 'DONE';
		MXPCREC.OA_IFACETIMESTAMP	:= SYSDATE;
		v_transseq := v_transseq + 1;
		MXPCREC.TRANSSEQ := v_transseq;
		MaxApiInsert.ins_rec(MXPCREC); -- Just change status
	ELSE
	    --
	    -- Get lines from ORACLE
	    --
		Put_In_Log(PrgName,'Getting po lines...');

		FOR pc_ln_rec IN po_ln(OFPOAPIRec.po_id) LOOP
			--
		 	-- Copying header information into new line.
			--
			MXPCLINEREC := MXPCREC;

			If pc_ln_rec.LAST_UPDATED_BY is not null then
				SELECT	user_name
				INTO	MXPCLINEREC.ENTERBY
				FROM	fnd_user
				WHERE	user_id = pc_ln_rec.LAST_UPDATED_BY;
			End If;

			MXPCLINEREC.ENTERDATE  	:=	NVL(pc_ln_rec.LAST_UPDATE_DATE,SYSDATE);

			IF pc_ln_rec.ITEM_ID IS NOT NULL THEN
	        	api_item_pkg.item_segs(pc_ln_rec.ITEM_ID,
	                               	MXPCLINEREC.ITEMNUM,
	                               	error_status);
			END IF;

			IF 	MXPCREC.CONTRACTTYPE = 'PLANNED' THEN
 				MXPCLINEREC.CONTRACTLINENUM	:=	'1'||LTRIM(TO_CHAR(pc_ln_rec.LINE_NUM,'0999'))
		                            		||LTRIM(TO_CHAR(pc_ln_rec.SHIPMENT_NUM,'0999'))
		                            		||LTRIM(TO_CHAR(pc_ln_rec.DISTRIBUTION_NUM,'0999'));
				IF pc_ln_rec.destination_organization_id IS NOT NULL THEN
				  	BEGIN
						SELECT	organization_code
						INTO	MXPCLINERec.OA_STORELOC
						FROM	mtl_parameters
						WHERE	organization_id = pc_ln_rec.destination_organization_id;
					EXCEPTION
						WHEN OTHERS THEN
							API_Put_Error_Sp(PrgName,'Error in organization_id...');
					END;
				END IF;

				MXPCLINERec.CL_DESCRIPTION_LD2 := MXPCLINERec.OA_STORELOC;

				IF pc_ln_rec.receiving_routing_id = 2 THEN
					MXPCLINERec.CL_INSPECTIONREQD	:=	1;
				ELSE
					MXPCLINERec.CL_INSPECTIONREQD	:=	0;
				END IF;

				MXPCLINEREC.ORDERQTY		:=	pc_ln_rec.QUANTITY_ORDERED - NVL(pc_ln_rec.pd_canqty,0);
				MXPCLINEREC.UNITCOST		:=	pc_ln_rec.UNIT_PRICE;
			ELSE
				MXPCLINEREC.CONTRACTLINENUM :=	pc_ln_rec.LINE_NUM;
				IF pc_ln_rec.ps_qty IS NOT NULL THEN
					MXPCLINEREC.ORDERQTY		:=		pc_ln_rec.ps_qty - NVL(pc_ln_rec.ps_canqty,0);
					MXPCLINEREC.UNITCOST		:=		pc_ln_rec.price_override;
				ELSE
					MXPCLINEREC.ORDERQTY		:=		pc_ln_rec.pl_qty;
					MXPCLINEREC.UNITCOST		:=		pc_ln_rec.UNIT_PRICE;
				END IF;
			END IF;

			MXPCLINEREC.ORDERUNIT  		:=	pc_ln_rec.UNIT_MEAS_LOOKUP_CODE;
			MXPCLINEREC.MODELNUM		:=	pc_ln_rec.VENDOR_PRODUCT_NUM;
			MXPCLINEREC.CL_DESCRIPTION	:=	pc_ln_rec.ITEM_DESCRIPTION;
			MXPCLINEREC.CL_CHGQTYONUSE	:=	1;

			IF pc_ln_rec.ORDER_TYPE_LOOKUP_CODE = 'QUANTITY' THEN
				IF MXPCLINEREC.ITEMNUM IS NOT NULL THEN
					MXPCLINEREC.LINETYPE := 'ITEM';
				ELSE
					MXPCLINEREC.LINETYPE := 'SERVICE';
				END IF;
			ELSIF pc_ln_rec.ORDER_TYPE_LOOKUP_CODE IN ( 'FIXED PRICE', 'AMOUNT' ) THEN
				MXPCLINEREC.LINETYPE := 'SERVICE';

				IF pc_ln_rec.ORDER_TYPE_LOOKUP_CODE = 'FIXED PRICE' THEN
					MXPCLINEREC.LINECOST := pc_ln_rec.AMOUNT;
				ELSE
					MXPCLINEREC.LINECOST := MXPCLINEREC.ORDERQTY * MXPCLINEREC.UNITCOST;
				END IF;

				MXPCLINEREC.ORDERQTY := Null;
				MXPCLINEREC.UNITCOST := Null;
			END IF;

			MXPCLINEREC.EXCHANGERATE2 := MaxUtlPack.GetExchangeRate(pc_ln_rec.mrc_rate);

			--
			-- Setting the Line statuses
			--
			IF NVL(pc_ln_rec.closed_flag,'*') = 'Y' OR NVL(pc_ln_rec.closed_code,'*') = 'FINALLY CLOSED' THEN
				MXPCLINEREC.CL_LINESTATUS	:=	'CAN';
			ELSIF NVL(pc_ln_rec.cancel_flag,'*') = 'Y' THEN
				MXPCLINEREC.CL_LINESTATUS	:=	'CAN';
			ELSE
				MXPCLINEREC.CL_LINESTATUS	:=	'APPR';
		    END IF;

	 		--
	    	--  Call user exit procedure for the lines
	    	--
			MOF_USR_PC_SP(POACTREC, POHEADREC, OFPOAPIRec, MXPCLINEREC, skip_line, SKIP_TRANSACTION);

	 		IF skip_transaction THEN
				RetProc(PrgName,'User Exit Returned skip_transaction = true');
				Return;
			END IF;

			IF NOT skip_line THEN
	 			MXPCLINEREC.OWNERSYSID 			:= 'OA';
				MXPCLINEREC.OA_IFACESTATUS		:= 'DONE';
				MXPCLINEREC.OA_IFACETIMESTAMP	:= SYSDATE;
				v_transseq 						:= v_transseq+1;
				MXPCLINEREC.TRANSSEQ 			:= v_transseq;
				MaxApiInsert.ins_rec(MXPCLINEREC);
			END IF;

		END LOOP;

	END IF;

/*
*	Insert into MXIN_INTER_TRANS table prior to interface table
*/
	IF v_transseq >0 THEN
    MaxUtlPack.PutApiTransIn(MaxutlPack.GetMaxOraCtl('EXTSYS'),
							 'MXPC_FROA12',
							 MXPCREC.TRANSID,
							 null);
	END IF;

	RetProc(PrgName,ProcMsg);


EXCEPTION
	WHEN MaxApiErrors.ApiError OR
    MaxApiErrors.ApiWarning OR
    MaxApiErrors.ApiException THEN
    	RAISE; -- re-raise if comes from lower
   	WHEN OTHERS THEN
    	MaxApiErrors.RaiseApiException(PrgName, procMsg);
END  API_PC_SP;
/
SHOW ERRORS

