CREATE OR REPLACE PROCEDURE API_RCVROT_SP
/*=========================================================================
* Licensed Materials - Property of IBM
*
* Restricted Materials of IBM
*
* 5724-M19
*
* (C) COPYRIGHT IBM CORP. 2008. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*==========================================================================
* Licensed Materials - Property of MRO Software
*
* Restricted Materials of MRO Software
*
* (C) COPYRIGHT MRO Software, Inc. 2001 - 2006. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with MRO Software, Inc.
*===========================================================================
*
* Filename:		OFRCVROT.SP
* Procedure:	API_RCVROT_SP
*
* Description:
*
*	Procedure to pass data coming from Oracle Financials to Maximo.
*	API that is called from ORACLE'S Receiving
*
* Public Functions:
*
*
* Modification History
*
*   15-Nov-2004 AKJ Created for Release 6.0
*   13-Jul-2005 AKJ Issue_85643 - Process REJECT transactions
*   25-Sep-2006 KM  Issue_06-18582 Mapped the column POSITEID
*   12-Jun-2008 AKJ Issue_08-16928 - Set the PO Line Num correctly for PO's not from Maximo.
* 	30-Oct-2008	FG	Issue_08-23591 - MX 5.2 to 7.1 MEAORA UPG - After upgrade the Receipts from Oracle for an Maximo PO errors out.
*==========================================================================*/
(
    RCVTRANREC           IN rcv_transactions%RowType,
    RCVSERREC            IN rcv_serial_transactions%RowType
) IS
	PrgName			 VARCHAR2(40) := 'API_RCVROT_SP';
	ProcMsg			 VARCHAR2(100);
	MAXRCVREC		 MXRCVROTITM_IFACE%ROWTYPE;

	CURSOR iface_c(ref_id MXRCVROTITM_IFACE.EXTERNALREFID%TYPE) IS
		SELECT transid FROM MXRCVROTITM_IFACE WHERE externalrefid = ref_id;

	shp_waybill_no	rcv_shipment_headers.WAYBILL_AIRBILL_NUM%Type;
	shp_pack_slip	rcv_shipment_lines.PACKING_SLIP%Type;
	shp_vnditm_no	rcv_shipment_lines.VENDOR_ITEM_NUM%Type;
	shp_unit_cost	rcv_shipment_lines.SHIPMENT_UNIT_PRICE%Type;
	shp_forg_id		rcv_shipment_lines.FROM_ORGANIZATION_ID%Type;

	rcv_po_no		po_headers.SEGMENT1%Type;
	rcv_po_source	po_headers.INTERFACE_SOURCE_CODE%Type;
	rcv_pl_no		po_lines.LINE_NUM%Type;
	rcv_itm_desc	po_lines.ITEM_DESCRIPTION%Type;
	rcv_order_type  po_line_types.ORDER_TYPE_LOOKUP_CODE%Type;
	rcv_ps_no	    po_line_locations.SHIPMENT_NUM%Type;
	rcv_ps_routingid po_line_locations.RECEIVING_ROUTING_ID%Type;
	rcv_pd_no		po_distributions.DISTRIBUTION_NUM%Type;
	rcv_pd_pono     po_distributions.ATTRIBUTE1%Type;
	rcv_pd_polineno po_distributions.ATTRIBUTE2%Type;

	rcv_rel_no		po_releases.RELEASE_NUM%Type;
	v_attribute15	po_releases.attribute15%Type;  -- holds maximo po/polinenum for a release po

	shp_forg_code	org_organization_definitions.organization_code%type;

	rcv_change_by	fnd_user.USER_NAME%Type;

	rcv_client_info varchar2(10);

	skip_transaction BOOLEAN;

	-- Issue_08-23591 Author: FG Date: Oct-30-2008 new variable with values in version 5.X - 6.X 
	rcv_creation_method	po_headers.DOCUMENT_CREATION_METHOD%Type;
	rcv_note	     po_headers.NOTE_TO_RECEIVER%Type;

BEGIN
	ProcMsg := 'ORAFIN Receipt API process for: '||RCVTRANREC.transaction_id;
	EnterProc(PrgName,ProcMsg);

	MaxDebugApi.Output('Getting receipt data...');

	IF IS_PARENT_REJECTED(RCVTRANREC.PARENT_TRANSACTION_ID) THEN
		-- Skip transaction when parent is Reject
        RetProc(PrgName, 'Parent transaction is REJECT - transaction skipped');
        Return;
	END IF;

	BEGIN
		MaxDebugApi.Output('Getting shipment data...');

		SELECT  WAYBILL_AIRBILL_NUM
		INTO	shp_waybill_no
		FROM    rcv_shipment_headers
		WHERE   SHIPMENT_HEADER_ID = RCVTRANREC.SHIPMENT_HEADER_ID;

		SELECT  PACKING_SLIP, VENDOR_ITEM_NUM, SHIPMENT_UNIT_PRICE, FROM_ORGANIZATION_ID
		INTO 	shp_pack_slip, shp_vnditm_no, shp_unit_cost, shp_forg_id
		FROM    rcv_shipment_lines
		WHERE   SHIPMENT_LINE_ID = RCVTRANREC.SHIPMENT_LINE_ID;

    EXCEPTION
		WHEN OTHERS THEN
			API_Put_Error_Sp(PrgName,'Error in shipment data...');
	END;

	IF RCVTRANREC.PO_LINE_LOCATION_ID IS NOT NULL THEN
		BEGIN
			MaxDebugApi.Output('Getting purchase order data...');

			-- Issue_08-23591 Author: FG Date: Oct-30-2008 add DOCUMENT_CREATION_METHOD and NOTE_TO_RECEIVER to the query
			SELECT  SEGMENT1, INTERFACE_SOURCE_CODE, DOCUMENT_CREATION_METHOD, NOTE_TO_RECEIVER
			INTO 	rcv_po_no, rcv_po_source, rcv_creation_method, rcv_note
			FROM    po_headers
			WHERE   PO_HEADER_ID = RCVTRANREC.PO_HEADER_ID;

			SELECT  pl.LINE_NUM, pl.ITEM_DESCRIPTION, plt.ORDER_TYPE_LOOKUP_CODE
			INTO 	rcv_pl_no, rcv_itm_desc, rcv_order_type
			FROM    po_lines pl, po_line_types plt
			WHERE   pl.PO_LINE_ID = RCVTRANREC.PO_LINE_ID
			AND		pl.LINE_TYPE_ID = plt.LINE_TYPE_ID;

			SELECT  SHIPMENT_NUM, RECEIVING_ROUTING_ID
			INTO 	rcv_ps_no, rcv_ps_routingid
			FROM    po_line_locations
			WHERE   LINE_LOCATION_ID = RCVTRANREC.PO_LINE_LOCATION_ID;

			-- 'REJECT' transactions do not specify a distribution, so we will take the first distribution
			IF RCVTRANREC.PO_DISTRIBUTION_ID IS NULL THEN
				DECLARE
					CURSOR c1 (p_line_location_id po_distributions.LINE_LOCATION_ID%Type) IS
						SELECT  DISTRIBUTION_NUM, ATTRIBUTE1, ATTRIBUTE2
						FROM    po_distributions
						WHERE   LINE_LOCATION_ID = p_line_location_id
						ORDER BY DISTRIBUTION_NUM;
				BEGIN
					OPEN c1(RCVTRANREC.PO_LINE_LOCATION_ID);
					FETCH c1 INTO rcv_pd_no, rcv_pd_pono, rcv_pd_polineno;
					CLOSE c1;
				END;
			ELSE
				SELECT  DISTRIBUTION_NUM, ATTRIBUTE1, ATTRIBUTE2
				INTO 	rcv_pd_no, rcv_pd_pono, rcv_pd_polineno
				FROM    po_distributions
				WHERE   PO_DISTRIBUTION_ID = RCVTRANREC.PO_DISTRIBUTION_ID;
			END IF;

	    EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in purchase order data...');
		END;
	END IF;

    If rcv_po_no IS Null then
       RetProc(PrgName,'Skipping transaction - Only accept PO Receipts');
       Return;
    End If;

	IF rcv_ps_routingid = 2 AND RCVTRANREC.TRANSACTION_TYPE = 'DELIVER' AND
			RCVTRANREC.INSPECTION_STATUS_CODE = 'REJECTED' THEN
       RetProc(PrgName,'Skipping transaction - Inspection Rejection not passed on DELIVER transaction');
       Return;
	END IF;

	IF RCVTRANREC.PO_RELEASE_ID IS NOT NULL THEN

		MaxDebugApi.Output('Getting po release...');

	  	BEGIN
			SELECT	pr.release_num, nvl(attribute15,'-1')
			INTO	rcv_rel_no, v_attribute15
			FROM	po_releases pr
			WHERE	pr.po_release_id = RCVTRANREC.PO_RELEASE_ID;
		EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in po releases...');
		END;
		--
		-- attribute15 will hold 'MAXIMO:' for a release PO
		--  that originated in MAXIMO
		If v_attribute15 LIKE 'MAXIMO:%' Then
		   rcv_po_no := rcv_pd_pono;
		ELSE
		   --  Do the conatentation of po and release_no only if the
		   --  blanket release originated in Oracle.
		   rcv_po_no := rcv_po_no || '-' || rcv_rel_no;
		End If;

	END IF;

	IF shp_forg_id IS NOT NULL THEN
	  	BEGIN
			SELECT	fd.organization_code
			INTO	shp_forg_code
			FROM	org_organization_definitions fd
			WHERE	fd.organization_id = shp_forg_id;
		EXCEPTION
			WHEN OTHERS THEN
				API_Put_Error_Sp(PrgName,'Error in from organization_id...');
		END;
	END IF;

	BEGIN
		SELECT	user_name
		  INTO	rcv_change_by
		  FROM	fnd_user
		 WHERE	user_id = RCVTRANREC.LAST_UPDATED_BY;
	EXCEPTION
		WHEN OTHERS THEN
			API_Put_Error_Sp(PrgName,'Error in user_id...');
	END;

	GET_OPERID_SP(rcv_client_info);

    MAXRCVREC.SITEID		  := RTRIM(rcv_client_info);
	MAXRCVREC.POSITEID		  := RTRIM(rcv_client_info);
	MAXRCVREC.ITEMNUM 	      := Null;
	MAXRCVREC.TOSTORELOC      := Null;

    IF RCVTRANREC.locator_id IS NULL THEN
    	MAXRCVREC.TOBIN       := RCVTRANREC.subinventory;
    ELSE
    	MAXRCVREC.TOBIN       := RCVTRANREC.subinventory || '-' || RCVTRANREC.locator_id;
    END IF;

	MAXRCVREC.PONUM	          := rcv_po_no;
	MAXRCVREC.TRANSDATE		  := RCVTRANREC.last_update_date;
	MAXRCVREC.ACTUALDATE	  := RCVTRANREC.transaction_date;

	If rcv_order_type = 'AMOUNT' Then
		MAXRCVREC.RECEIPTQUANTITY := Null;
		MAXRCVREC.RECEIVEDUNIT	  := RCVTRANREC.unit_of_measure;
	Else
		MAXRCVREC.RECEIPTQUANTITY := ABS(RCVTRANREC.primary_quantity);
		MAXRCVREC.RECEIVEDUNIT	  := RCVTRANREC.primary_unit_of_measure;
	End If;

	If RCVTRANREC.transaction_type IN('DELIVER','REJECT')
			AND NVL(RCVTRANREC.primary_quantity,1) >= 0 Then
		MAXRCVREC.ISSUETYPE	  := 'RECEIPT';
	Else
		MAXRCVREC.ISSUETYPE	  := 'RETURN';
	End If;

	MAXRCVREC.ACTUALCOST	  := NVL(shp_unit_cost, RCVTRANREC.po_unit_price);
	MAXRCVREC.INVOICENUM	  := Null;
	MAXRCVREC.REJECTCODE	  := RCVTRANREC.rma_reference;

	If rcv_ps_routingid = 2 Then
		MAXRCVREC.INSPECTED   := 1;
		If RCVTRANREC.TRANSACTION_TYPE = 'DELIVER' Then
			MAXRCVREC.REJECTQTY	  := 0;
			MAXRCVREC.ACCEPTEDQTY := ABS(RCVTRANREC.primary_quantity);
		Else  -- TRANSACTION_TYPE = 'REJECT'
			MAXRCVREC.REJECTQTY	  := ABS(RCVTRANREC.primary_quantity);
			MAXRCVREC.ACCEPTEDQTY := 0;
		End If;
	Else
		MAXRCVREC.INSPECTED   := 0;
		MAXRCVREC.REJECTQTY	  := Null;
		MAXRCVREC.ACCEPTEDQTY := Null;
	End If;

	MAXRCVREC.CONVERSION	  := 1;
	MAXRCVREC.REFWO			  := Null;
	MAXRCVREC.ENTERBY		  := rcv_change_by;
	MAXRCVREC.OUTSIDE		  := Null;
	MAXRCVREC.ISSUETO		  := Null;
	MAXRCVREC.PACKINGSLIPNUM  := NVL(shp_pack_slip,shp_waybill_no);

	-- Issue_08-23591 Author: FG Date: 30-Oct-2008 Include rcv_creation_method check
	If rcv_po_source is null Then
		If (upper(rcv_note) like upper('Approved MAXIMO Purchase Order%') and  nvl(rcv_creation_method,'*') <> 'COPY_DOCUMENT') Then
			rcv_po_source := 'MAXIMO';
		End If;
	End If;	

	If rcv_pd_polineno IS NOT NULL AND rcv_po_source = 'MAXIMO' then
	   MAXRCVREC.POLINENUM := rcv_pd_polineno;
	Else
	   MAXRCVREC.POLINENUM := '1'||LTRIM(TO_CHAR(rcv_pl_no,'0999'))||LTRIM(TO_CHAR(rcv_ps_no,'0999'))||LTRIM(TO_CHAR(rcv_pd_no,'0999'));
	End If;

	MAXRCVREC.ISSUE			  := Null;
	MAXRCVREC.REQUESTEDBY	  := Null;
	MAXRCVREC.TOTALCURBAL	  := Null;
	MAXRCVREC.OLDAVGCOST	  := Null;
	MAXRCVREC.GLDEBITACCT	  := Null;
	MAXRCVREC.GLCREDITACCT	  := Null;
	MAXRCVREC.FINANCIALPERIOD := Null;
	MAXRCVREC.CURRENCYCODE	  := RCVTRANREC.currency_code;
	MAXRCVREC.EXCHANGERATE	  := NVL(RCVTRANREC.currency_conversion_rate,1);
    MAXRCVREC.EXCHANGERATE2	  := MaxUtlPack.GetExchangeRate(RCVTRANREC.mrc_currency_conversion_rate);
	MAXRCVREC.MANUFACTURER	  := Null;
	MAXRCVREC.MODELNUM		  := shp_vnditm_no;
	MAXRCVREC.CURRENCYUNITCOST:= Null;
	MAXRCVREC.CURRENCYLINECOST:= NVL(NVL(shp_unit_cost,RCVTRANREC.po_unit_price),0) *
								 NVL(RCVTRANREC.currency_conversion_rate,1) *
								 NVL(RCVTRANREC.quantity,1);
	MAXRCVREC.LOCATION		  := Null;
	MAXRCVREC.DESCRIPTION	  := rcv_itm_desc;
	MAXRCVREC.REMARK		  := RCVTRANREC.comments;
	MAXRCVREC.FROMSTORELOC	  := UPPER(shp_forg_code);
	MAXRCVREC.FROMBIN		  := Null;
	MAXRCVREC.QTYHELD		  := Null;
	MAXRCVREC.FROMLOT		  := Null;
	MAXRCVREC.TOLOT 		  := RCVTRANREC.vendor_lot_num;
	MAXRCVREC.LOADEDCOST	  := Null;
	MAXRCVREC.TAX1CODE	 	  := Null;
	MAXRCVREC.TAX1		 	  := Null;
	MAXRCVREC.TAX2CODE	 	  := Null;
	MAXRCVREC.TAX2		 	  := Null;
	MAXRCVREC.TAX3CODE	 	  := Null;
	MAXRCVREC.TAX3		 	  := Null;
	MAXRCVREC.TAX4CODE	 	  := Null;
	MAXRCVREC.TAX4		 	  := Null;
	MAXRCVREC.TAX5CODE	 	  := Null;
	MAXRCVREC.TAX5		  	  := Null;
	MAXRCVREC.PRORATED	  	  := 0;
	MAXRCVREC.PRORATECOST  	  := Null;
	MAXRCVREC.STATUS		  := Null;
	MAXRCVREC.STATUSDATE	  := Null;
	MAXRCVREC.STATUSCHANGEBY  := Null;
	MAXRCVREC.CURBAL		  := 0;
	MAXRCVREC.EXTERNALREFID   := RCVTRANREC.transaction_id;
	MAXRCVREC.SENDERSYSID     := Null;
	MAXRCVREC.TRANSSEQ		  := 1;
	MAXRCVREC.OA_IFACESTATUS  := 'DONE';
	MAXRCVREC.OA_IFACETIMESTAMP	:= SYSDATE;

	MAXRCVREC.ASSETNUM 		  := RCVSERREC.serial_num;
    MAXRCVREC.AS_SITEID		  := RTRIM(rcv_client_info);
	MAXRCVREC.MATRECTRANSID	  := 0;

	/*
	*   Call user exit procedure
	*/
	MOF_USR_RCVROT_SP(RCVTRANREC, RCVSERREC, MAXRCVREC, skip_transaction);

	IF skip_transaction THEN
		RetProc(PrgName,'User Exit Returned skip_transaction = TRUE');
		Return;
	END IF;

	MaxDebugApi.Output('Getting transid...');

	OPEN iface_c(MAXRCVREC.EXTERNALREFID);

	FETCH iface_c INTO MAXRCVREC.TRANSID;

	IF iface_c%NOTFOUND THEN
		/*
		*	Insert into MXIN_INTER_TRANS table prior to interface table
		*/
		SELECT maxifacetransseq.nextval INTO MAXRCVREC.TRANSID FROM DUAL;
	    MaxUtlPack.PutApiTransIn(MaxutlPack.GetMaxOraCtl('EXTSYS'), 'MXRCVROTITM_FROA12', MAXRCVREC.TRANSID, null);
	END IF;

	IF iface_c%ISOPEN THEN
		CLOSE iface_c;
	END IF;

	/*
	*	Insert into MAXIMO RCVROT Interface table
	*/
	MaxApiInsert.ins_rec(MAXRCVREC);

	RetProc(PrgName,ProcMsg);

EXCEPTION
   WHEN MaxApiErrors.ApiError OR
         MaxApiErrors.ApiWarning OR
         MaxApiErrors.ApiException THEN
      RAISE; -- re-raise if comes from lower
   WHEN OTHERS THEN
      MaxApiErrors.RaiseApiException(PrgName, procMsg);
END  API_RCVROT_SP;
/
SHOW ERRORS