CREATE OR REPLACE PROCEDURE MXE_RCV_SP
/*=========================================================================
* Licensed Materials - Property of IBM
*
* Restricted Materials of IBM
*
* 5724-M19
*
* (C) COPYRIGHT IBM CORP. 2008. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
* =========================================================================
* Licensed Materials - Property of MRO Software
*
* Restricted Materials of MRO Software
*
* (C) COPYRIGHT MRO Software, Inc. 2001 - 2006. All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with MRO Software, Inc.
*==========================================================================
*
* Filename:	ORCVX.SP
* Procedure:	MXE_RCV_SP
*
* Description:
*
*	Procedure to pass data coming from Maximo to external system.
*
* Modification History
*
*	12-Apr-1996	Pradeep Kumar	Created.
*   05-Jul-2000 Tom Sarasin     only send receipts for POs to Oracle
*	15-May-2001 KM	            To use ITMORG if there is no inv org tied to the shipto
*								and added Shipped_date to the Interface table.
*   06-Jun-2001 KM              Updated the insertion of interface tables
*   04-Jan-2002 KM              Upgrade to 5.0
*   15-Jul-2002 AKJ             Changed format of date string to ISO 8601 format
*   25-Nov-2002 AKJ             Don't set Receipt_num value
*   21-Apr-2003 AKJ             Issue_032938 - Raise warning for negative quantity.
*	14-Dec-2004 KM				Upgraded to 6.0 Release
*	03-May-2005 KM				Issue_82868 - shipment_num to null
*   03-Aug-2005 AKJ				Issue_86357 - Pass inspection status to Oracle
*   13-Sep-2005 KM              Issue_88003 For SPORDER items not populating the
*                               OraRCVTransIntRec.ITEM_ID column.
*   08-Feb-2006 AKJ  			Issue_84602 - Added item id lookup using all segments.
* 	22-Sep-2006 SKM				Issue_84060 - Added code for returns. Right now rotating item return is not
* 									implemented, also partial insp can't be done on lot control item.
*	30-Oct-2007	FG				Issue_ 06-18162 - Receipt with lot numbers fails with "exact fetch returns more than requested number of rows" message.
*	28-Apr-2008 FG				Issue_08-14892 - Fixed Price and Rate Based Receipt Lines error during Import to Oracle R12
*   10-Jul-2008 HF				Issue #08-18150 - Populating RCV_TRANSACTIONS_INTERFACE amount field to link receipt to po (blank OACATXREF SERVICE entry in Maximo)			
*   11-Jul-2008 FR				Issue #08-18499 - Populating RCV_TRANSACTIONS_INTERFACE category id with value from po line when line type is FIXED PRICE SERVICE/TEMP LABOR	
* 	05-Nov-2008 FR				Issue #08-23739 - Populate PONUM with the right value if it came from a Blanket release.
* 	07-Nov-2008 FG				Issue #08-23873 - MX 5.2 to 7.1 MEAORA UPG - After upgrade the Receipts from Maximo for a CAI PO does not have DELIVER transaction in Oracle. 
* 													Receipt is not complete in Oracle Apps. It is also a 5.x to 6 upgrade bug in orcvx10.sp.
*   27-Nov-2008 FG				Issue #08-25050 - MX 6.2 to 7.1 MEAORA UPG - Getting 'APIException: no data found' in MXRECEIPT_IFACE table, when doing receipt from Release PO in MAXIMO. 		
*==========================================================================*/
(
	IntRec IN OUT MXRECEIPT_IFACE%RowType
)IS
	PrgName		     	VARCHAR2(40) := 'MXE_RCV_SP';
	ProcMsg		      	VARCHAR2(100);
	OraRCVHdIntRec	  	RCV_HEADERS_INTERFACE%ROWTYPE;
	OraRCVTransIntRec 	RCV_TRANSACTIONS_INTERFACE%ROWTYPE;
	OraRCVAcceptRec 	RCV_TRANSACTIONS_INTERFACE%ROWTYPE;
	OraRCVDeliverRec 	RCV_TRANSACTIONS_INTERFACE%ROWTYPE;
	OraRCVRejectRec 	RCV_TRANSACTIONS_INTERFACE%ROWTYPE;
	OraRCVRetToVendRec 	RCV_TRANSACTIONS_INTERFACE%ROWTYPE;
	Error_Status  	  	NUMBER;
	clnt_info        	VARCHAR2(64);
	vENTERBYID	     	NUMBER;
    vVENDOR          	MXRECEIPT_IFACE.OA_VENDOR%TYPE;
    vORGANIZATION_ID 	NUMBER;
	vPONUM			 	MXRECEIPT_IFACE.PONUM%TYPE;
	vPOHEADERID		 	OraRCVTransIntRec.PO_HEADER_ID%TYPE;
	vPOLINEID			OraRCVTransIntRec.PO_LINE_ID%TYPE;
	vPOLINETYPE			PO_LINES.LINE_TYPE_ID%TYPE; -- Issue #08-18150
	vCATEGORY_ID		PO_LINES.CATEGORY_ID%TYPE; -- Issue #08-18499
	vLOCATION_ID		OraRCVHdIntRec.LOCATION_ID%TYPE;
	vGROUP_ID			OraRCVHdIntRec.GROUP_ID%TYPE;
	vlinenum         	VARCHAR2(15);
	vROUTING_ID			PO_LINE_LOCATIONS.RECEIVING_ROUTING_ID%Type;
	vSHIPMENT_NUM		PO_LINE_LOCATIONS.SHIPMENT_NUM%Type;
	v_shipped_date	 	DATE;
	skip_transaction 	BOOLEAN;
	v_group_found		BOOLEAN := False;
	v_receipt_found		BOOLEAN := False;
	v_parent_id			RCV_TRANSACTIONS.TRANSACTION_ID%TYPE;
	vCommentPart		VARCHAR2(60);
	vTEMP_SUM			NUMBER;
	vQtyToRet			NUMBER;
	vReturnableQty		NUMBER;
	vQty			NUMBER;
	vReturnedAllQty		boolean;
	
	vProdTransId		MTL_TRANSACTION_LOTS_INTERFACE.PRODUCT_TRANSACTION_ID%Type;
	TYPE OraRCVTransPart IS RECORD (
		transaction_id		RCV_TRANSACTIONS.TRANSACTION_ID%Type,
		quantity			RCV_TRANSACTIONS.QUANTITY%Type);
	OraRCVTransPartRec		OraRCVTransPart;
		
	TYPE OraRCVTransIntPart IS RECORD (
		interface_transaction_id	RCV_TRANSACTIONS_INTERFACE.INTERFACE_TRANSACTION_ID%Type,
		quantity					RCV_TRANSACTIONS_INTERFACE.QUANTITY%Type);
	OraRCVTransIntPartRec		OraRCVTransIntPart;
	vSqlString				VARCHAR2(1000);
	TYPE refCursorTyp IS REF CURSOR;
    baseTableCur refCursorTyp;
    intTableCur	refCursorTyp;
    
    OraRCVTransRec		RCV_TRANSACTIONS%ROWTYPE;
BEGIN
	ProcMsg := ' PO #:' || IntRec.PONUM;
	EnterProc(PrgName,ProcMsg);

    --
	-- Setup the Operating Unit Id for ORACLE Financials
	--
	clnt_info := IntRec.SITEID;
	SET_OPERID_SP(clnt_info);
	MaxDebugApi.Output('back from set op id: '|| clnt_info);
	
	IF IntRec.MATRECTRANSID is not null THEN
		-- Issue #06-18162 Author: FG Date: 30-Oct - 2007
		IF IntRec.ISSUETYPE = 'RECEIPT' THEN
			vCommentPart := 'MAXIMO RECEIPT ' || NVL(IntRec.RECEIPTREF, IntRec.MATRECTRANSID);
		ELSE
			vCommentPart := 'MAXIMO RECEIPT ' || IntRec.MATRECTRANSID || ' MAXIMO REF ' || NVL(IntRec.RECEIPTREF, IntRec.MATRECTRANSID);
		END IF;
	ELSE
		vCommentPart := 'MAXIMO RECEIPT SERVRECTRANS '||IntRec.SERVRECTRANSID;
	END IF;

	/*
	 * Author: FG Date: 17 Dec 2007 Description: in OA12 CAI support was discontinued, and all the PO will be treated as standard
	 * Validate PONUM
	 * 
	 * Author: FR Date: 05 Nov 2008
	 * Fix for Issue#08-23739 - Receipts from Maximo for an Oracle owned Release fails in MXRECEIPT_IFACE table.
	 * Populate PONUM with the right value if it came from a Blanket release.
	 * 
	 * Author: FG Date: 27 Nov 2008
	 * Issue #08-25050
	 * Exclude elsif clause for receiving maximo owned release. This causes 'no data found' exception because adapter try to find shipments
	 * for blanket po.
	 */
	If (IntRec.OA_POTYPE = 'REL' and nvl(IntRec.OA_OWNERSYSID,'x') = 'OA') then
       	-- this is a blanket release originated in Oracle
	    vPONUM := Substr(IntRec.PONUM,1,Instr(IntRec.PONUM,'-')-1);
	else 
		-- this is a standard PO
		vPONUM := IntRec.PONUM;
	End If;
	MaxDebugApi.Output('vPONUM: '||vPONUM );
	/**
	 * End of Fix for Issue#08-23739 - Receipts from Maximo for an Oracle owned Release fails in MXRECEIPT_IFACE table.
	 */
	
	of_id_pkg.get_id_sp('POHEADERID', vPONUM, clnt_info, vPOHEADERID, Error_Status);
	IF Error_Status != 0 THEN
		IntRec.OA_IFACEMESSAGE := 'Invalid PO Number';
		MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
	END IF;
	If nvl(IntRec.OA_OWNERSYSID,'x') = 'OA' Then         -- PO originated from Oracle
	   vlinenum := to_char(IntRec.polinenum);
	   OraRCVTransIntRec.DOCUMENT_LINE_NUM		:= to_number(substr(vlinenum,2,4));
	   vSHIPMENT_NUM							:= to_number(substr(vlinenum,6,4));
	Else
	   OraRCVTransIntRec.DOCUMENT_LINE_NUM		:= IntRec.polinenum;
	   vSHIPMENT_NUM							:= 1;
	End If;

	-- Check if this recipt has inspection required routing
	/*	Issue #08-23873 - Author FG - Date 07-Nov-2008
		Description: 	Receiving_routing_id field comes empty from 5.x 6.x maximo owner po's
						Hence, deliver transaction is not created in rcv_transactions_interface.
						When no value is found, vROUTING_ID is set to 3, as there weren't any inspection process in previous
						versions, then all the receiving transaction can be treated as direct delivery. */	
	BEGIN
		SELECT nvl(ps.receiving_routing_id, 3) INTO vROUTING_ID
		  FROM po_lines pl, po_line_locations ps
		 WHERE pl.po_header_id = vPOHEADERID AND
		 	   pl.line_num = OraRCVTransIntRec.DOCUMENT_LINE_NUM AND
		 	   pl.po_line_id = ps.po_line_id AND
		 	   ps.shipment_num = vSHIPMENT_NUM;
	EXCEPTION
		WHEN OTHERS THEN
			IntRec.OA_IfaceMessage := 'PO Shipment not found:'||vPOHEADERID||
				'*'||OraRCVTransIntRec.DOCUMENT_LINE_NUM||'*'||vSHIPMENT_NUM;
			MaxApiErrors.RaiseApiException(PrgName, IntRec.OA_IfaceMessage); 
	END;

	/*
	* Check if this receipt is already in interface table.  Because we will get 
	* Accept and Reject Inspection records seperately we should put these records
	* under the same receipt.
	* Added vROUTING_ID = 2 because non inspection required receipt we get only one transaction.
	*/
	v_group_found := False;
	IF not (IntRec.ISSUETYPE='RETURN') and vROUTING_ID = 2 THEN
	
		/* IN THE ABOVE CONDITION THIS WAS THERE In the and IntRec.matrectransid is not null */
		BEGIN
			select *
				into OraRCVTransIntRec
		  		from RCV_TRANSACTIONS_INTERFACE
		  		where COMMENTS like vCommentPart ||':%' AND
			   			TRANSACTION_TYPE = 'RECEIVE' AND
			   			PROCESSING_STATUS_CODE = 'PENDING';

			v_group_found := True;
		EXCEPTION
			WHEN OTHERS THEN
				v_group_found := False;
    	END;
    END IF;
    
    /* If the receipt in not in the interface table then we might have to enter a record.
     * So evaluate some of the information we need to insert a record to interface table.
     */
    IF NOT v_group_found THEN
		of_id_pkg.get_id_sp('RCV_GROUP_ID',
				    NULL,
				    clnt_info,
				    vGROUP_ID,
				    Error_Status);
		If Error_Status != 0 then
			IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction group id';
			MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
		End If;
		MaxDebugApi.Output('after group id');

		of_id_pkg.get_id_sp('SHIPTO',
					    IntRec.OA_SHIPTO,
						clnt_info,
					    vLOCATION_ID,
					    Error_Status);
		MaxDebugApi.Output(PrgName||'LOCATION_ID: '||vLOCATION_ID);
	
		If Error_Status != 0 then
			IntRec.OA_IFACEMESSAGE := 'Invalid location-To. It is not Location in ORAFIN.';
			MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
		End If;
		MaxDebugApi.Output('after location id');

		/*
		*	Get USERID and EMPLOYEE_ID for ENTERBY
		*/
		If IntRec.ENTERBY is NOT NULL THEN
	
			of_id_pkg.get_id_sp('USRID', IntRec.ENTERBY, clnt_info, vENTERBYID, Error_Status);
			If Error_Status != 0 then
				IntRec.OA_IFACEMESSAGE := 'Modified By User: ' || IntRec.ENTERBY || ' is not a valid user in Oracle Financials';
				MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
			End If;
	
			of_id_pkg.get_id_sp('EMPIDFORUSER', IntRec.ENTERBY, clnt_info, OraRCVHdIntRec.EMPLOYEE_ID, Error_Status);
			If Error_Status != 0 then
				IntRec.OA_IFACEMESSAGE := 'Modified By User: ' || IntRec.ENTERBY || ' is not valid employee in Oracle Financials';
				MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
			End If;
		End If;
		
		MaxDebugApi.Output('after user id');
	End If;
	
	-- Check if this receipt is already in the base tables
	-- Added vROUTING_ID = 2 because non inspection required receipt we get only one transaction.
    v_receipt_found := False;
    IF (IntRec.ISSUETYPE <> 'RETURN') and (vROUTING_ID = 2) THEN
      /* IN THE ABOVE CONDITION THIS WAS THERE In the and IntRec.matrectransid is not null */
		BEGIN
			select transaction_id
		  		into v_parent_id
		  		from RCV_TRANSACTIONS
		 		where COMMENTS like vCommentPart ||':%' AND
		       			TRANSACTION_TYPE = 'RECEIVE';

			v_receipt_found := True;
		EXCEPTION
  	  		WHEN OTHERS THEN
				v_receipt_found := False;
    	END;
    
    END IF;
	

	/**
	 * Author: FGROCHA Date: 03 Jul 2008
	 * Fix for issue 08-17374-Receipt Return from MAXIMO to Oracle for MAXIMO Owned Items failing.
	 * The organization id was not being populated because this code block was inside a not satisfied
	 * if clause.
	 */
	/*
	*  Validate that storeroom and bin are entered when receiving an inventory item
	*/
	If IntRec.TOSTORELOC IS NULL then -- direct issue/service
		-- use the inv org tied to the shipto
  	  	of_id_pkg.get_id_sp('DESTID',
        	    			IntRec.OA_SHIPTO,
							clnt_info,
		    				vORGANIZATION_ID,
		     				Error_Status);
	    If Error_Status != 0 then
			IntRec.OA_IFACEMESSAGE := 'Invalid Ship-to Organization';
			MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
		End If;

		-- If there is no inv org tied to the shipto, then use Default ORG
      	IF vORGANIZATION_ID is null THEN
         	of_id_pkg.get_id_sp('DEFAULTORG',
            					null,
            					clnt_info,
            					vORGANIZATION_ID,
            					Error_Status);

         	IF Error_Status != 0 THEN
            	IntRec.OA_IfaceMessage := 'Invalid Financials System Parameters';
            	MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IfaceMessage);
         	END IF;
      	END IF;
		MaxDebugApi.Output(PrgName||' vORGANIZATION_ID: '||vORGANIZATION_ID);
	Else
		of_id_pkg.get_id_sp('ORGID', IntRec.TOSTORELOC, clnt_info, vORGANIZATION_ID, Error_Status);
     	If Error_Status != 0 then
     	   	ProcMsg := 'Location: '||IntRec.TOSTORELOC||' is not a valid Organization in Oracle Financials';
     	   	MaxApiErrors.RaiseApiError(PrgName, ProcMsg);
        End If;
		MaxDebugApi.Output(PrgName||' vORGANIZATION_ID of TOSTORELOC: '||vORGANIZATION_ID);
	END IF;
	/**
	 * End of Fix for issue 08-17374-Receipt Return from MAXIMO to Oracle for MAXIMO Owned Items failing.
	 */


	/*
	* We need to build the header record if the receipt has not already been put into the
	* interface table or loaded into the base table. Build Header record. For returns
	* we don't need a header.
	*/
	IF (NOT (v_group_found OR v_receipt_found)) and (IntRec.ISSUETYPE <> 'RETURN') THEN
		/*
		*	Get unique identifiers for RCV_HEADERS_INTERFACE and RCV_TRANSACTIONS_INTERFACE
		*/
		of_id_pkg.get_id_sp('RCV_HEADER_ID',
						    NULL,
						    clnt_info,
						    OraRCVHdIntRec.HEADER_INTERFACE_ID,
						    Error_Status);
		If Error_Status != 0 then
			IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction header';
			MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
		End If;
		MaxDebugApi.Output('after header id');

		of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', 
							NULL, 
							clnt_info, 
							OraRCVTransIntRec.INTERFACE_TRANSACTION_ID, 
							Error_Status);
		If Error_Status != 0 then
			IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
			MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
		End If;
	
		/*
		*	Seperating Vendor from Operating Unit ID
		*/
		vVENDOR := IntRec.OA_VENDOR;
	
		IF INSTR(vVENDOR,'-') != 0 THEN
			vVENDOR := SUBSTR(vVENDOR, INSTR(vVENDOR, '-') + 1);
			MaxDebugApi.Output('Vendor after operunitid is removed:'||vVENDOR);
		End If;
	
		MaxDebugApi.Output('after vendor');
	
		IF vVENDOR IS NOT NULL THEN
		  IF INSTR(vVENDOR,'-') != 0 THEN
			 of_id_pkg.get_id_sp('VNDID',
					    SUBSTR(vVENDOR,0,INSTR(vVENDOR,'-')-1),
					    clnt_info,
					    OraRCVHdIntRec.VENDOR_ID,
					    Error_Status);
			 If Error_Status != 0 then
				IntRec.OA_IFACEMESSAGE := 'Invalid vendor header';
				MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
			 End If;
		  ELSE
			 of_id_pkg.get_id_sp('VNDID',
					    		 vVENDOR,
					    		 clnt_info,
					    		 OraRCVHdIntRec.VENDOR_ID,
					    		 Error_Status);
			 If Error_Status != 0 then
				IntRec.OA_IFACEMESSAGE := 'Invalid vendor';
				MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
			 End If;
		  End If;
		END IF;
		MaxDebugApi.Output('after vendor id');

		/*
		*	Assign other values to RCV_HEADERS_INTERFACE table
		*/
		OraRCVHdIntRec.GROUP_ID					:=	vGROUP_ID;
		OraRCVHdIntRec.EXPECTED_RECEIPT_DATE 	:=	NVL(IntRec.OA_REQUIREDDATE, SYSDATE);
	    OraRCVHdIntRec.SHIPPED_DATE 		 	:=	LEAST(sysdate, OraRCVHdIntRec.EXPECTED_RECEIPT_DATE);
		OraRCVHdIntRec.PACKING_SLIP 			:=	IntRec.PACKINGSLIPNUM;
		OraRCVHdIntRec.PROCESSING_STATUS_CODE	:=	'PENDING';
		OraRCVHdIntRec.RECEIPT_SOURCE_CODE		:=	'VENDOR';
		OraRCVHdIntRec.TRANSACTION_TYPE			:=	'NEW';
		OraRCVHdIntRec.LAST_UPDATED_BY			:=	vENTERBYID;
		OraRCVHdIntRec.LAST_UPDATE_LOGIN		:=	vENTERBYID;
		OraRCVHdIntRec.LAST_UPDATE_DATE			:=	SYSDATE;
		OraRCVHdIntRec.CREATION_DATE			:=	NVL(IntRec.ACTUALDATE,IntRec.TRANSDATE);
		OraRCVHdIntRec.CREATED_BY	      		:=	vENTERBYID;
		OraRCVHdIntRec.VALIDATION_FLAG			:=	'Y';
		OraRCVHdIntRec.SHIP_TO_ORGANIZATION_ID 	:=	vORGANIZATION_ID;
		OraRCVHdIntRec.LOCATION_ID				:=	vLOCATION_ID;
	END IF;
	
	/*
	* We need to build the receipt record if the receipt has not already been put into the
	* interface table.  We will not save this receipt if it is already in the base table,
	* but we will use it as a template for the inspection and deliver records.
	*/
	IF NOT v_group_found THEN
		/*
		*	Assign values to RCV_TRANSACTIONS_INTERFACE table
		*/
	    IF IntRec.OA_ITEM_ID IS NOT NULL THEN
	        OraRCVTransIntRec.ITEM_ID := IntRec.OA_ITEM_ID;
		ELSIF IntRec.LINETYPE = 'SPORDER' THEN
      		OraRCVTransIntRec.ITEM_ID := Null;
	    ELSIF IntRec.ItemNum IS NOT NULL THEN
            OraRCVTransIntRec.ITEM_ID := api_item_pkg.GetItemId(vORGANIZATION_ID, IntRec.ItemNum);
	    END IF;		
		OraRCVTransIntRec.GROUP_ID			        :=	vGROUP_ID;
		OraRCVTransIntRec.HEADER_INTERFACE_ID		:=	OraRCVHdIntRec.HEADER_INTERFACE_ID;
		OraRCVTransIntRec.PACKING_SLIP 				:=	IntRec.PACKINGSLIPNUM;
	    OraRCVTransIntRec.CATEGORY_ID 				:=	IntRec.OA_CATEGORYID;
		OraRCVTransIntRec.SubInventory 				:=	null;
		OraRCVTransIntRec.LOCATOR_ID 				:=	IntRec.OA_LOCATOR_ID;
		OraRCVTransIntRec.LAST_UPDATED_BY		    :=	vENTERBYID;
		OraRCVTransIntRec.LAST_UPDATE_DATE		    :=	SYSDATE;
		OraRCVTransIntRec.CREATION_DATE			    :=	NVL(IntRec.ACTUALDATE,IntRec.TRANSDATE);
		OraRCVTransIntRec.CREATED_BY	     	    :=	vENTERBYID;
	   	OraRCVTransIntRec.TRANSACTION_TYPE		    :=	'RECEIVE';
		OraRCVTransIntRec.TRANSACTION_DATE		    :=	LEAST(IntRec.TRANSDATE, SYSDATE);
		OraRCVTransIntRec.PROCESSING_STATUS_CODE	:=	'PENDING';
		OraRCVTransIntRec.PROCESSING_MODE_CODE		:=	'BATCH';
		OraRCVTransIntRec.TRANSACTION_STATUS_CODE	:=	'PENDING';
		OraRCVTransIntRec.QUANTITY			        :=	NVL(IntRec.OA_INSP_RCV_QTY, NVL(IntRec.RECEIPTQUANTITY, IntRec.QTYTORECEIVE));

		-- Issue #08-18150
		OraRCVTransIntRec.AMOUNT			        :=	IntRec.amttoreceive;  


		/*
		 * Fix for Issue #08-18499 - Receipt Outbound for Fixed Price Services fails during import
		 * Populates category id with the category id from the po line 
		 */ 
		SELECT	line_type_id, category_id INTO vPOLINETYPE, vCATEGORY_ID
		FROM	po_lines
		WHERE	po_header_id = vPOHEADERID AND
                line_num = OraRCVTransIntRec.DOCUMENT_LINE_NUM;
                                                

        IF (vPOLINETYPE = 5 OR vPOLINETYPE = 6) THEN -- Fixed Price Temp Labor and Fixed Price Services
                OraRCVTransIntRec.CATEGORY_ID := vCATEGORY_ID;
        END IF;
		/*
		 * End of Fix for Issue #08-18499 - Receipt Outbound for Fixed Price Services fails during import
		 */ 

		OraRCVTransIntRec.UNIT_OF_MEASURE		    :=	IntRec.RECEIVEDUNIT;
		OraRCVTransIntRec.ITEM_DESCRIPTION		    :=	IntRec.DESCRIPTION;
		OraRCVTransIntRec.AUTO_TRANSACT_CODE 		:= 	'RECEIVE';
		OraRCVTransIntRec.RECEIPT_SOURCE_CODE		:=	'VENDOR';
		OraRCVTransIntRec.SOURCE_DOCUMENT_CODE		:=	'PO';
		OraRCVTransIntRec.VALIDATION_FLAG		    :=	'Y';
		OraRCVTransIntRec.LAST_UPDATE_LOGIN		    :=	vENTERBYID;
		
		/* Needed when we will fix Issue # 06-18162
		IF IntRec.MATRECTRANSID is not null THEN
			OraRCVTransIntRec.COMMENTS := SUBSTR('MAXIMO RECEIPT '|| IntRec.MATRECTRANSID ||':'||IntRec.Remark, 1, 240);
		ELSE
			OraRCVTransIntRec.COMMENTS := SUBSTR('MAXIMO RECEIPT SERVRECTRANS '|| IntRec.SERVRECTRANSID||':'||IntRec.Remark, 1, 240);
		END IF;
	    */
		OraRCVTransIntRec.COMMENTS                  :=	SUBSTR(vCommentPart ||':'||IntRec.Remark, 1, 240);
		OraRCVTransIntRec.DOCUMENT_NUM              :=	vPONUM;
	    OraRCVTransIntRec.DELIVER_TO_LOCATION_ID	:=	vLOCATION_ID;
	END IF;

	If not (IntRec.ISSUETYPE='RETURN') then
		/*
		 * If inspection routing then build the accept and deliver record 
		 * or reject and return to vendor record.
		 */
		IF vROUTING_ID = 2 THEN
			/*  For material receipt, accept record's ISSUETYPE is TRANSFER
			 *  For service receipt,  accept record's ISSUETYPE is RECEIVE and RECEIVE record
			 *  is created for both accept and reject. So we check the ACCEPTEDQTY to see
			 *  if any qty is accepted.
			 */
	  		IF  IntRec.ISSUETYPE='TRANSFER' or IntRec.ACCEPTEDQTY >0 THEN 
				-- Build ACCEPT Record
				OraRCVAcceptRec := OraRCVTransIntRec;
				of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', NULL, clnt_info, OraRCVAcceptRec.INTERFACE_TRANSACTION_ID, Error_Status);
				If Error_Status != 0 then
					IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
					MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
				End If;
				OraRCVAcceptRec.TRANSACTION_TYPE := 'ACCEPT';
				OraRCVAcceptRec.INSPECTION_STATUS_CODE := 'ACCEPTED';
				OraRCVAcceptRec.QUANTITY := NVL(IntRec.RECEIPTQUANTITY,IntRec.ACCEPTEDQTY );
				OraRCVAcceptRec.AUTO_TRANSACT_CODE := Null;
				OraRCVAcceptRec.PARENT_INTERFACE_TXN_ID := OraRCVTransIntRec.INTERFACE_TRANSACTION_ID;
 
				OraRCVAcceptRec.PARENT_TRANSACTION_ID := v_parent_id;

				-- Build DELIVER Record
				OraRCVDeliverRec := OraRCVTransIntRec;
				of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', NULL, clnt_info, OraRCVDeliverRec.INTERFACE_TRANSACTION_ID, Error_Status);
				If Error_Status != 0 then
					IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
					MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
				End If;
				
				/*
				Author: FG Date: 26-Dec-2007
				Description:
				Set the subinventory for the inspected and accepted items to the destination subiventory
				*/
				OraRCVDeliverRec.SUBINVENTORY := IntRec.OA_SUBINV_CODE;				
				
				OraRCVDeliverRec.TRANSACTION_TYPE := 'DELIVER';
				OraRCVDeliverRec.QUANTITY := OraRCVAcceptRec.QUANTITY;
				OraRCVDeliverRec.AUTO_TRANSACT_CODE := Null;
				OraRCVDeliverRec.PARENT_INTERFACE_TXN_ID := OraRCVAcceptRec.INTERFACE_TRANSACTION_ID;
			END IF;
						
			/* Create REJECT and RETURN TO VENDOR record.
			 *
			 * For material receipt, reject record's ISSUETYPE is REJECT.
			 * Note that for service receipt we skip the REJECT record in Maximo side erpexit.
			 * For service record we see the REJECTQTY in the accept record.
			 */
			IF  IntRec.ISSUETYPE='REJECT' or IntRec.REJECTQTY >0 THEN
				-- Build Inspection Record
				OraRCVRejectRec := OraRCVTransIntRec;
				of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', NULL, clnt_info, OraRCVRejectRec.INTERFACE_TRANSACTION_ID, Error_Status);
				If Error_Status != 0 then
					IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
					MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
				End If;
				OraRCVRejectRec.TRANSACTION_TYPE := 'REJECT';
				OraRCVRejectRec.INSPECTION_STATUS_CODE := 'REJECTED';
				OraRCVRejectRec.QUANTITY := nvl(IntRec.RECEIPTQUANTITY,IntRec.REJECTQTY);
				OraRCVRejectRec.AUTO_TRANSACT_CODE := Null;
				OraRCVRejectRec.PARENT_INTERFACE_TXN_ID := OraRCVTransIntRec.INTERFACE_TRANSACTION_ID;
				OraRCVRejectRec.PARENT_TRANSACTION_ID := v_parent_id;

				-- Build Deliver Record
				OraRCVRetToVendRec := OraRCVTransIntRec;
				of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', NULL, clnt_info, OraRCVRetToVendRec.INTERFACE_TRANSACTION_ID, Error_Status);
				If Error_Status != 0 then
					IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
					MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
				End If;
				OraRCVRetToVendRec.TRANSACTION_TYPE := 'RETURN TO VENDOR';
				OraRCVRetToVendRec.QUANTITY := OraRCVRejectRec.QUANTITY;
				OraRCVRetToVendRec.AUTO_TRANSACT_CODE := Null;
				OraRCVRetToVendRec.PARENT_INTERFACE_TXN_ID := OraRCVRejectRec.INTERFACE_TRANSACTION_ID;
			END IF;
		
		ELSE
			-- This is not a inspection routing receipt.
			-- so create a deliver record.
			
			-- Issue#08-14892 - When PO is service type with no inspection, auto-deliver process will be used
			IF IntRec.SERVRECTRANSID is not null THEN
				OraRCVTransIntRec.SUBINVENTORY 			:= IntRec.OA_SUBINV_CODE;
				OraRCVTransIntRec.AUTO_TRANSACT_CODE 	:= 	'DELIVER';
			ELSE
				OraRCVDeliverRec := OraRCVTransIntRec;
				of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', NULL, clnt_info, OraRCVDeliverRec.INTERFACE_TRANSACTION_ID, Error_Status);
				If Error_Status != 0 then
					IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
					MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
				End If;
				
				/*
				Author: FG Date: 26-Dec-2007
				Description:
				Set the subinventory for the inspected and accepted items to the destination subiventory
				*/
				OraRCVDeliverRec.SUBINVENTORY := IntRec.OA_SUBINV_CODE;
				
				OraRCVDeliverRec.TRANSACTION_TYPE := 'DELIVER';
				OraRCVDeliverRec.AUTO_TRANSACT_CODE := Null;
				OraRCVDeliverRec.PARENT_INTERFACE_TXN_ID := OraRCVTransIntRec.INTERFACE_TRANSACTION_ID;
			END IF;


		END IF;
		
		/* Needed when we will fix Issue # 06-18162
		IF NOT (v_group_found OR v_receipt_found) THEN
			OraRCVTransIntRec.COMMENTS := SUBSTR(vCommentPart ||':'||IntRec.Remark, 1, 240);
		END IF;
		*/

		/*
		*	call user exit before inserting
		*/
		MXE_USR_RCV_SP(IntRec, OraRCVHdIntRec, OraRCVTransIntRec, OraRCVAcceptRec, 
				   OraRCVDeliverRec, OraRCVRejectRec, OraRCVRetToVendRec, skip_transaction);  
	
		--
		-- check for SKIP transaction
		--
		IF skip_transaction THEN
			MaxDebugApi.Output('User Exit Returned skip_transaction = true');
			Return;
		END IF;

		/*
		*   	Populate Oracle Financials RCV Interface Table
		*/
		IF NOT (v_group_found OR v_receipt_found) THEN
			MaxDebugApi.Output('Insert Shipment into RCV_HEADERS_INTERFACE');
			MaxApiInsert.ins_rec(OraRCVHdIntRec);
	
			MaxDebugApi.Output('Insert Receipt into RCV_TRANSACTIONS_INTERFACE');
			MaxApiInsert.ins_rec(OraRCVTransIntRec);

			-- Issue#08-14892 - If receiving line is not of service type, insert the deliver record
			-- otherwise there is no need to insert it because of auto-deliver feature used for service type lines
			IF vROUTING_ID <> 2 and IntRec.SERVRECTRANSID is null THEN
				MaxDebugApi.Output('Insert Shipment into RCV_HEADERS_INTERFACE');
				MaxApiInsert.ins_rec(OraRCVDeliverRec);
			END IF;

		END IF;

		IF OraRCVAcceptRec.INTERFACE_TRANSACTION_ID is not null THEN
			MaxDebugApi.Output('Insert Inspection into RCV_TRANSACTIONS_INTERFACE');
			MaxApiInsert.ins_rec(OraRCVAcceptRec);


			MaxDebugApi.Output('Insert Deliver into RCV_TRANSACTIONS_INTERFACE');
			MaxApiInsert.ins_rec(OraRCVDeliverRec);

		END IF;

		IF OraRCVRejectRec.INTERFACE_TRANSACTION_ID is not null THEN
			MaxDebugApi.Output('Insert Reject into RCV_TRANSACTIONS_INTERFACE');
			MaxApiInsert.ins_rec(OraRCVRejectRec);

			MaxDebugApi.Output('Insert Return to vendor into RCV_TRANSACTIONS_INTERFACE');
			MaxApiInsert.ins_rec(OraRCVRetToVendRec);
		END IF;
	ELSE
		-- This is a RETURN
		if( IntRec.MATRECTRANSID is null) then
 			vQtyToRet := IntRec.ACCEPTEDQTY * -1;
 		else
 			vQtyToRet := IntRec.RECEIPTQUANTITY * -1;
 		end if;
 		-- There is no Header record for RETURN
 		OraRCVTransIntRec.HEADER_INTERFACE_ID := null;
 		BEGIN
			-- Issue#08-21436 Changed where statement because of maximo po line num transformation during PO Inbound
        	SELECT po_line_id INTO vPOLINEID
				FROM po_lines
				WHERE po_header_id = vPOHEADERID AND 
			 		line_num = OraRCVTransIntRec.DOCUMENT_LINE_NUM;
 		EXCEPTION
 			WHEN OTHERS THEN
				IntRec.OA_IfaceMessage := 'PO_LINE record not found for po_header_id:' || vPOHEADERID ||
												'  pl.line_num:' || IntRec.POLINENUM;
				MaxApiErrors.RaiseApiException(PrgName, IntRec.OA_IfaceMessage);
		END; 
    	vReturnedAllQty:=false;  -- This is set to true when we have created return for all records.
    	/* In Maximo the RETURN is not always tied to one receipt. Say we have two receipt
    	 * of 2 items each. Then the user can create one RETURN of 4 items. Where as in Oracle apps
    	 * receipt is done against DELIVER. So for one maximo Return we can have multiple oracle returns.
    	 * The following code finds all DELIVER record against which we can do a return.
    	 */ 
    	IF(IntRec.TOLOT is not null) THEN
     		vSqlString := 'SELECT a.transaction_id, a.quantity '
 							|| ' FROM rcv_transactions a, mtl_transaction_lot_numbers b '
							|| ' WHERE a.po_header_id=' || vPOHEADERID || ' AND a.po_line_id= ' || vPOLINEID
				    	  	|| ' AND a.transaction_type=''DELIVER'' AND a.transaction_id=b.product_transaction_id ' 
				    	  	|| ' AND b.product_code = ''RCV'' and b.LOT_NUMBER = ''' || IntRec.TOLOT || '''';
    	ELSE
    		vSqlString := 'SELECT transaction_id, quantity  '
 							|| ' FROM rcv_transactions '
							|| ' WHERE po_header_id=' || vPOHEADERID || ' AND po_line_id= ' || vPOLINEID
				    	  	|| ' AND transaction_type=''DELIVER'''; 
		END IF; 
 		OPEN baseTableCur FOR vSqlString;
		LOOP
        	FETCH baseTableCur INTO     OraRCVTransPartRec;  -- fetch next row
        	EXIT WHEN baseTableCur%NOTFOUND  or vReturnedAllQty;
        	vReturnableQty := OraRCVTransPartRec.quantity;
        	-- How many qty already returned against this DELIVER record?
        	SELECT nvl(sum(quantity),0) INTO vQty 
 				FROM rcv_transactions 
				WHERE parent_transaction_id= OraRCVTransPartRec.transaction_id
					  and transaction_type='RETURN TO RECEIVING';
			vReturnableQty := vReturnableQty - vQty;
			-- There may be some RETURN in the interface table, consider those.
			SELECT nvl(sum(quantity),0) INTO vQty 
 				FROM rcv_transactions_interface 
				WHERE parent_transaction_id= OraRCVTransPartRec.transaction_id
					  and transaction_type='RETURN TO VENDOR';
			vReturnableQty := vReturnableQty - vQty;
			if(vReturnableQty >0) THEN
				-- We can return vReturnableQty against this deliver record.
				OraRCVRetToVendRec := OraRCVTransIntRec;
				of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', NULL, clnt_info, OraRCVRetToVendRec.INTERFACE_TRANSACTION_ID, Error_Status);
				If Error_Status != 0 then
					IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
					MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
				End If;
				OraRCVRetToVendRec.TRANSACTION_TYPE := 'RETURN TO VENDOR';
					
				IF(vQtyToRet >= vReturnableQty) THEN
					OraRCVRetToVendRec.QUANTITY := vReturnableQty;
					if (vQtyToRet = vReturnableQty) then
				    	vReturnedAllQty := true;
					end if;
					vQtyToRet := vQtyToRet - vReturnableQty;	
				ELSE
					OraRCVRetToVendRec.QUANTITY := vQtyToRet;
					vReturnedAllQty := true;
					vQtyToRet := 0;
				END IF;
					
				OraRCVRetToVendRec.AUTO_TRANSACT_CODE := Null;
				OraRCVRetToVendRec.parent_transaction_id:= OraRCVTransPartRec.transaction_id;
				
				MXE_USR_RCV_SP(IntRec, OraRCVHdIntRec, OraRCVRetToVendRec, OraRCVAcceptRec, 
						OraRCVDeliverRec, OraRCVRejectRec, OraRCVRetToVendRec, skip_transaction);  
	
				IF skip_transaction THEN
				    MaxDebugApi.Output('User Exit Returned skip_transaction = true');
				ELSE
					MaxApiInsert.ins_rec(OraRCVRetToVendRec);
				END IF;
			END IF;		
		END LOOP;
		IF  not vReturnedAllQty  then	
			/* There are still few qty to return. There may be some deliver record
			 * in the interface table. Return against those deliver records.
			 */	
	
    		IF(IntRec.TOLOT is not null) THEN
     			vSqlString := 'SELECT a.interface_transaction_id, a.quantity '
     		
 	 						|| ' FROM rcv_transactions_interface a, mtl_transaction_lots_interface b '				
	 						|| ' WHERE a.document_num=''' || IntRec.PONUM 
	 						|| ''' AND a.document_line_num=' || IntRec.POLINENUM 
					      	|| ' AND a.transaction_type=''DELIVER'''
					      	|| ' AND a.interface_transaction_id=b.product_transaction_id ' 
				    	  	|| ' AND b.product_code = ''RCV''' 
					        || ' and LOT_NUMBER = ''' || IntRec.TOLOT || '''';
    		ELSE
    			vSqlString := 'SELECT interface_transaction_id, quantity  '
 	 						|| ' FROM rcv_transactions_interface '					
	 						|| ' WHERE document_num=''' || IntRec.PONUM 
	 						|| ''' AND document_line_num=' || IntRec.POLINENUM 
					      	|| ' AND transaction_type=''DELIVER''';
			END IF;   		
	
	        OPEN intTableCur FOR vSqlString;		      
			LOOP
	        	FETCH intTableCur INTO     OraRCVTransIntPartRec;  -- fetch next row
	        	EXIT WHEN intTableCur%NOTFOUND  or vReturnedAllQty;
	        	vReturnableQty := OraRCVTransIntPartRec.quantity;

				SELECT nvl(sum(quantity),0) INTO vQty 
	 				FROM rcv_transactions_interface 
					WHERE PARENT_INTERFACE_TXN_ID = OraRCVTransIntPartRec.INTERFACE_TRANSACTION_ID
						  and transaction_type='RETURN TO VENDOR';
				vReturnableQty := vReturnableQty - vQty;
				if(vReturnableQty >0) THEN
					OraRCVRetToVendRec := OraRCVTransIntRec;
					of_id_pkg.get_id_sp('RCV_TRANSACTION_ID', NULL, clnt_info, OraRCVRetToVendRec.INTERFACE_TRANSACTION_ID, Error_Status);
					If Error_Status != 0 then
						IntRec.OA_IFACEMESSAGE := 'Invalid receive transaction id';
						MaxApiErrors.RaiseApiError(PrgName, IntRec.OA_IFACEMESSAGE);
					End If;
					OraRCVRetToVendRec.TRANSACTION_TYPE := 'RETURN TO VENDOR';	
					IF(vQtyToRet >= vReturnableQty) THEN
						OraRCVRetToVendRec.QUANTITY := vReturnableQty;
						if (vQtyToRet = vReturnableQty) then
						    vReturnedAllQty := true;
						end if;
						vQtyToRet := vQtyToRet - vReturnableQty;	
					ELSE
						OraRCVRetToVendRec.QUANTITY := vQtyToRet;
						vReturnedAllQty := true;
						vQtyToRet := 0;
					END IF;
					OraRCVRetToVendRec.AUTO_TRANSACT_CODE := Null;
					OraRCVRetToVendRec.PARENT_INTERFACE_TXN_ID := OraRCVTransIntPartRec.INTERFACE_TRANSACTION_ID;
					
					MXE_USR_RCV_SP(IntRec, OraRCVHdIntRec, OraRCVRetToVendRec, OraRCVAcceptRec, 
							OraRCVDeliverRec, OraRCVRejectRec, OraRCVRetToVendRec, skip_transaction);  
	
					IF skip_transaction THEN
					    MaxDebugApi.Output('User Exit Returned skip_transaction = true');
					ELSE
						MaxApiInsert.ins_rec(OraRCVRetToVendRec);
					END IF;
				
					
				END IF;		
			END LOOP;
		END IF;
		IF  not vReturnedAllQty  then
		    IntRec.OA_IfaceMessage := 'More quantity to return then received. Can not return '||
			                          		vQtyToRet|| ' unit(s) of this';
			MaxApiErrors.RaiseApiException(PrgName, IntRec.OA_IfaceMessage);
		END IF;
	END IF;
	RetProc(PrgName,ProcMsg);

EXCEPTION
	WHEN MaxApiErrors.ApiError
	OR MaxApiErrors.ApiWarning
	OR MaxApiErrors.ApiException THEN
		Raise;			 --- Re-raise if comes from lower
	WHEN OTHERS THEN
		MaxApiErrors.RaiseApiException(PrgName,ProcMsg);
END MXE_RCV_SP;
/
SHOW ERRORS
